-- check_patt.vhd
-- jamieson olsen <jamieson@fnal.gov>
-- updated 31 October 2001
-- updated 25 March 2004 -- remove the input register, absorb into the link deskew circuit.

-- checks for a 28-bit "walking ones" sequence.
--
--	M                          L
--	7654321098765432109876543210
--	...........................1
--	..........................1.
--	.........................1..
--	........................1...
--	.......................1....
--	......................1.....
--	.....................1......
--	....................1.......
--	...................1........
--	..................1.........
--	.................1..........
--	................1...........
--	...............1............
--	..............1.............
--	.............1..............
--	............1...............
--	...........1................
--	..........1.................
--	.........1..................
--	........1...................
--	.......1....................
--	......1.....................
--	.....1......................
--	....1.......................
--	...1........................
--	..1.........................
--	.1..........................
--	1...........................
--	xxxxxxxxxxxxxxxxxxxxxxxxxxxx  <- this word contains the color and sector fields!  don't check!
--	...........................1
--	..........................1.
--	.........................1..
--	........................1...
--	.......................1....
--	......................1.....
--	.....................1......
--	....................1.......
--	...................1........
--	..................1.........

-- 	checking is started by monitoring bits [27..24] and waiting for the pattern 1000.
--  the next word is skipped, and the next 28 words are checked for the walking 1's pattern.
--  the above pattern MUST be transmitted continuously.

-- 	if no EoP is detected within 64 clock cycles, generate an error.


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity check_patt is
    port (
        clock: 		in 	std_logic;						-- clock 53 MHz
        reset: 		in 	std_logic;						-- active low, sync
		d:			in	std_logic_vector(27 downto 0);	-- data pattern
		error:		out	std_logic						-- set if error in pattern
    );
end check_patt;

architecture check_patt_arch of check_patt is

	-- define registers

--	signal a_reg:		std_logic_vector(27 downto 0);
	signal b_reg:		std_logic_vector(27 downto 0);
	signal xor_reg:		std_logic_vector(27 downto 0);
	signal sum_reg:		std_logic_vector( 3 downto 0);
	signal err_reg:		std_logic;
	signal eop_reg:		std_logic;						-- set when a_reg the last word of the pattern
	signal dly_eop_reg:	std_logic;
	signal cnt_reg:		std_logic_vector(5 downto 0);
	

	-- define signals

	signal temp_xor:		std_logic_vector(27 downto 0);
	signal temp_sum:		std_logic_vector( 3 downto 0);
	signal temp_err:		std_logic;
	signal temp_eop:		std_logic;	
	signal temp_wdog:		std_logic;

	-- declare components...
	
begin

	-- combinatorial stuff...

		temp_eop	<= 	'1' when (d(27 downto 24)="1000") else '0';

		error 		<= err_reg;

		temp_xor( 0)	<= d( 0) xor b_reg( 0);
		temp_xor( 1)	<= d( 1) xor b_reg( 1);
		temp_xor( 2)	<= d( 2) xor b_reg( 2);
		temp_xor( 3)	<= d( 3) xor b_reg( 3);
		temp_xor( 4)	<= d( 4) xor b_reg( 4);
		temp_xor( 5)	<= d( 5) xor b_reg( 5);
		temp_xor( 6)	<= d( 6) xor b_reg( 6);
		temp_xor( 7)	<= d( 7) xor b_reg( 7);
		temp_xor( 8)	<= d( 8) xor b_reg( 8);
		temp_xor( 9)	<= d( 9) xor b_reg( 9);
		temp_xor(10)	<= d(10) xor b_reg(10);
		temp_xor(11)	<= d(11) xor b_reg(11);
		temp_xor(12)	<= d(12) xor b_reg(12);
		temp_xor(13)	<= d(13) xor b_reg(13);
		temp_xor(14)	<= d(14) xor b_reg(14);
		temp_xor(15)	<= d(15) xor b_reg(15);
		temp_xor(16)	<= d(16) xor b_reg(16);
		temp_xor(17)	<= d(17) xor b_reg(17);
		temp_xor(18)	<= d(18) xor b_reg(18);
		temp_xor(19)	<= d(19) xor b_reg(19);
		temp_xor(20)	<= d(20) xor b_reg(20);
		temp_xor(21)	<= d(21) xor b_reg(21);
		temp_xor(22)	<= d(22) xor b_reg(22);
		temp_xor(23)	<= d(23) xor b_reg(23);
		temp_xor(24)	<= d(24) xor b_reg(24);
		temp_xor(25)	<= d(25) xor b_reg(25);
		temp_xor(26)	<= d(26) xor b_reg(26);
		temp_xor(27)	<= d(27) xor b_reg(27);
	
		temp_sum(0)		<= 	xor_reg(0) or xor_reg(1) or xor_reg(2) or xor_reg(3) or
							xor_reg(4) or xor_reg(5) or xor_reg(6);
	
		temp_sum(1)		<= 	xor_reg(7) or xor_reg(8) or xor_reg(9) or xor_reg(10) or
							xor_reg(11) or xor_reg(12) or xor_reg(13);
	
		temp_sum(2)		<= 	xor_reg(14) or xor_reg(15) or xor_reg(16) or xor_reg(17) or
							xor_reg(18) or xor_reg(19) or xor_reg(20);
	
		temp_sum(3)		<= 	xor_reg(21) or xor_reg(22) or xor_reg(23) or xor_reg(24) or
							xor_reg(25) or xor_reg(26) or xor_reg(27);

		temp_err		<= 	sum_reg(3) or sum_reg(2) or sum_reg(1) or sum_reg(0);

		temp_wdog		<=	'1' when (cnt_reg(5 downto 0)="111111") else '0';

		
	-- sequential stuff...

		check_pattern: process(clock)
		 begin
			if rising_edge(clock) then
				if (reset='0') then
					-- a_reg(27 downto 0) 		<= X"0000000";
					b_reg(27 downto 0) 		<= X"0000000";
					xor_reg(27 downto 0) 	<= X"0000000";
					sum_reg(3 downto 0)		<= "0000";
 					cnt_reg(5 downto 0)		<= "000000";
					err_reg 				<= '0';
					eop_reg					<= '0';
					dly_eop_reg				<= '0';
				else
					-- a_reg(27 downto 0)		<= d(27 downto 0);
					eop_reg 				<= temp_eop;
					dly_eop_reg				<= eop_reg;
					b_reg(27 downto 1)		<= b_reg(26 downto 0);		
					b_reg(0)				<= eop_reg;
					xor_reg(27 downto 0)	<= temp_xor(27 downto 0);
					err_reg 				<= temp_err or temp_wdog;
			
					if (eop_reg='1') then										-- end of pattern seen, reset the watchdog timer
						cnt_reg(5 downto 0)		<= "000000";
					else														-- increment the watchdog timer...
						cnt_reg(5 downto 0) 	<= cnt_reg(5 downto 0) + '1';
					end if;

					if (dly_eop_reg='1') then
						sum_reg(3 downto 0)		<= "0000";
					else
						sum_reg(3 downto 0)		<= temp_sum(3 downto 0);
					end if;					


				end if;
			end if;
		end process;
		
end check_patt_arch;
