-- link_dskew.vhd
-- 7 jan 2002
-- jamieson olsen <jamieson@fnal.gov>

-- store link data in a dual-port ram.  pull out and flatten it into an array of
-- 196 bits.  minimizes the logic clocked by the link_clock.  refer to my engineering note
-- 2002-01-07A.

-- the 7-word x 28-bit frame is flattened like this: q(0) is the lsb of the first word.  q(27) is
-- the msb of the first word.  q(28) is the lsb of the second word.  etc...

-- timing: ideally, CLEAR should be asserted as the 2nd word is being clocked into LINK_DATA().
-- this will keep the read pointer trailing the write pointer by ~1 clock cycle; this minimizes
-- the latency though this module while providing a phase margin of about 18ns between the board
-- CLOCK and the LINK_CLK.

-- if link skew is really bad, assert CLEAR as the 3rd word is being clocked into LINK_DATA().
-- this will increase latency by another clock cycle but will increase the phase margin to 36ns.

-- revised 24 March 2004: cleanup for XST

library ieee,mylib;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use mylib.dfea_pack.all;
entity link_dskew is
 port (
		link_data: 	in 	std_logic_vector(27 downto 0);
		link_clk:	in 	std_logic;

		reset:		in		std_logic;
		clock:		in		std_logic;
		clear:		in		std_logic;			-- active high, clear the read pointer, sync to CLOCK
-- inject should start at the same clock cycle as clear Aug-14-04
      inject:		in		std_logic;        -- inject fake data
		q: 			out	std_logic_vector(195 downto 0);
-- link data to be sent to other chips through ddr_link
		link_out:	out	std_logic_vector(27 downto 0);
      perr:			out	std_logic; -- pattern check error
      clk_err:		out	std_logic; -- link_clk period error
      sync_err:	out	std_logic; -- input frame to system frame error
      period_err:	out	std_logic; -- start bit period error
-- this can be used to observe eof timing. eof should come later and the
-- minimum separation is 2ns
      diag:			out	std_logic;
-- backplane control signals, added for run2b jul-06-2004
      wr_fake:		in		std_logic;
      rd_fake:		in		std_logic;
      w_data:		in		std_logic_vector(15 downto 0);
      r_data:		out	std_logic_vector(15 downto 0);
      addr:			in		std_logic_vector(15 downto 0)
    );
end link_dskew;


architecture link_dskew_arch of link_dskew is

	-- define registers...

--	signal link_data_reg: 	std_logic_vector(27 downto 0);
	signal link_data_reg: 	std_logic_vector(31 downto 0);
--	signal link_addr_reg: 	std_logic_vector(3 downto 0);
	signal link_addr_reg: 	std_logic_vector(8 downto 0);

--	signal read_addr_reg:	std_logic_vector(  3 downto 0);
	signal read_addr_reg:	std_logic_vector(  8 downto 0);
	signal shift_reg:		std_logic_vector(167 downto 0);
	signal shadow_reg:		std_logic_vector(195 downto 0);
	signal shadow_en_reg:	std_logic;

	-- define signals...

--	signal dp_ram_out:		std_logic_vector(27 downto 0);
	signal dp_ram_out:		std_logic_vector(31 downto 0);
	signal temp_shadow_en:	std_logic;
	-- signal logic:			std_logic_vector(1 downto 0);
	signal rd_data:			std_logic_vector(15 downto 0);
	-- declare components...
		
		component check_patt
		    port (
		        clock: 		in 	std_logic;						-- clock 53 MHz
		        reset: 		in 	std_logic;						-- active low, sync
				d:			in	std_logic_vector(27 downto 0);	-- data pattern
				error:		out	std_logic						-- set if error in pattern
		    );
		end component;
component fd
  port (d : in std_logic;
        c : in std_logic;
		  q : out std_logic);
end component;
component rom16x1
  generic(init : bit_vector := x"e");
  port (
    A0 : in std_logic;
    A1 : in std_logic;
    A2 : in std_logic;
    A3 : in std_logic;
    O : out std_logic
  );
end component;
component lut2
  generic(init : bit_vector := x"e");
  port (
    I0 : in std_logic;
    I1 : in std_logic;
    O : out std_logic
  );
end component;
signal link_div4: std_logic_vector(1 downto 0);
signal clk_div2: std_logic;
signal clk_cntr: std_logic_vector(3 downto 0);
signal clk_sync: std_logic_vector(3 downto 0);
signal d_sync_window: std_logic;
signal sync_window: std_logic;
signal d_sync_err: std_logic;
signal sync_err_i: std_logic;
signal injectq: std_logic;
signal addr_q: std_logic_vector(3 downto 0);
signal ctrl_addr: std_logic_vector(9 downto 0);
signal fake_addr: std_logic_vector(8 downto 0);
signal fake_data: std_logic_vector(31 downto 0);
signal ctrl_dout: std_logic_vector(15 downto 0);
signal d_link_eof: std_logic;
signal link_eof: std_logic;
signal global_bof: std_logic;
attribute RLOC: string;
attribute RLOC of i_bof: label is "x0y2";
attribute RLOC of i_eof: label is "x0y0";
attribute RLOC of i_diag: label is "x0y2";
attribute RLOC of i_sync_window: label is "x0y0";
attribute RLOC of i_sync_err: label is "x0y2";
--attribute RLOC of global_bof: signal is "x0y2";
--attribute RLOC of link_eof: signal is "x0y0";
--attribute RLOC of sync_window: signal is "x0y0";
--attribute RLOC of sync_err: signal is "x0y2";
begin
i_diag: LUT2 port map(I0 => link_eof, I1 => global_bof, o => diag);
process(clock)
begin
  if(clock'event and clock = '1')then
--	 global_bof <= clear;
--	 sync_err_i <= (not sync_window and clear) or (sync_err_i and not clear);
  end if;
end process;
process(link_clk)
begin
  if(link_clk'event and link_clk = '1')then
--	 link_eof <= link_addr_reg(2) and link_addr_reg(0);
--	 sync_window <= not link_addr_reg(0) and (link_addr_reg(2) xnor link_addr_reg(1));
  end if;
end process;
--i_diag: rom16x1 port map(a0 => link_eof, a1 => global_bof, a2 => gnd, a3 => gnd, o => diag);
i_eof: fd port map(d => d_link_eof, c => link_clk, q => link_eof);
i_bof: fd port map(d => clear, c => clock, q => global_bof);
-- this window spans two clock cycle starting from eof
d_sync_window <= not link_addr_reg(0) and (link_addr_reg(2) xnor link_addr_reg(1));
i_sync_window: fd port map(d => d_sync_window, c => link_clk, q => sync_window);
-- if clear comes too early or too late relative to link side eof, error is set
sync_err <= sync_err_i;
d_sync_err <= not sync_window when clear = '1' else sync_err_i;
i_sync_err: fd port map(d => d_sync_err, c => clock, q => sync_err_i);
	-- combinatorial stuff...
	d_link_eof <= link_addr_reg(2) and link_addr_reg(0);
	link_out <= dp_ram_out(27 downto 0);
-- block ram has synchronous read, takes one more clock cycle jul-6-2004 S.X.Wu
	temp_shadow_en 	<= '1' when (read_addr_reg(2 downto 0)="110") else '0';
--	temp_shadow_en 	<= '1' when (read_addr_reg(3 downto 0)="0101") else '0';

	q(195 downto 0)	<= shadow_reg(195 downto 0);

	--logic(0) <= '0';
	--logic(1) <= '1';
-- link input register removed to compensate for clock cycles lost in link_data
-- fanouts jul-06-2004 S.X.Wu 
			link_data_reg(27 downto 0) <= link_data(27 downto 0);
			link_data_reg(31 downto 28) <= "0000";
			link_addr_reg(8 downto 3) <= "000000";
-- read_addr_reg(3) = injectq makes inject independent of link side
			read_addr_reg(8 downto 3) <= "00000" & injectq;
	-- sequential stuff...

	link_side: process(link_clk)
	 begin
		if rising_edge(link_clk) then

-- link input register removed to compensate for clock cycles lost in link_data
-- fanouts jul-06-2004 S.X.Wu 
--			link_data_reg(27 downto 0) <= link_data(27 downto 0);
--			if (link_data_reg(0)='1') then
			if (link_data_reg(0)='1' or link_addr_reg(2 downto 1) = "11") then
				link_addr_reg(2 downto 0) <= "000";
			else
				link_addr_reg(2 downto 0) <= link_addr_reg(2 downto 0) + 1;
			end if;
		end if;
	end process;

	global_side: process(clock)
	 begin

		if rising_edge(clock) then
			if (clear='1') then
				read_addr_reg(2 downto 0) <= "000";
			else
				read_addr_reg(2 downto 0) <= read_addr_reg(2 downto 0) + 1;
			end if;
			shadow_en_reg 				<= temp_shadow_en;
			shift_reg(27 downto 0) 		<= dp_ram_out(27 downto 0);
			shift_reg(55 downto 28) 	<= shift_reg(27 downto 0);
			shift_reg(83 downto 56) 	<= shift_reg(55 downto 28);
			shift_reg(111 downto 84) 	<= shift_reg(83 downto 56);
			shift_reg(139 downto 112) 	<= shift_reg(111 downto 84);
			shift_reg(167 downto 140) 	<= shift_reg(139 downto 112);

			if (shadow_en_reg='1') then

				shadow_reg(27 downto 0) 	<= shift_reg(167 downto 140);		-- first word of frame
				shadow_reg(55 downto 28) 	<= shift_reg(139 downto 112);		-- 2nd word
				shadow_reg(83 downto 56) 	<= shift_reg(111 downto  84);		-- 3rd word
				shadow_reg(111 downto 84) 	<= shift_reg( 83 downto  56);		-- 4th word
				shadow_reg(139 downto 112) 	<= shift_reg( 55 downto  28);		-- 5th word
				shadow_reg(167 downto 140) 	<= shift_reg( 27 downto   0);		-- 6th word
				shadow_reg(195 downto 168)	<= dp_ram_out(27 downto 0);		-- 7th word (last)

			end if;

		end if;
	end process;


-- instantiate components...
-- input fake aug-06-2004 S.X.Wu
		fake_dpram: RAMB16_S18_S36
		   port map(
				dia => w_data,
			   dib => gnd32,
				dipa => gnd2,
				dipb => gnd4,
				ena => vcc,
				enb => vcc,
				wea => wr_fake,
				web => gnd,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock,
				addra => ctrl_addr,
				addrb => fake_addr,
				doa => ctrl_dout,
				dob => fake_data);
ctrl_addr <= "000000" & addr(3 downto 0) when wr_fake = '0' else "000000" & addr_q;
r_data <= ctrl_dout when rd_fake = '1' else (others => 'Z');
fake_addr(8 downto 3) <= "000000";
process(clock)
begin
  if(clock'event and clock = '1')then
    injectq <= inject;
	 addr_q <= addr(3 downto 0);
    if(inject = '0' or fake_addr(2 downto 1) = "11")then
	   fake_addr(2 downto 0) <= "000";
	 else
	   fake_addr(2 downto 0) <= fake_addr(2 downto 0) + 1;
	 end if;
  end if;
end process;
-- block ram replaced distributed ram jul-06-2004 S.X.Wu
		link_dpram: RAMB16_S36_S36
		   port map(
			   dia => link_data_reg,
				dib => fake_data,
				dipa => gnd4,
				dipb => gnd4,
				ena => vcc,
				enb => vcc,
				wea => vcc,
				web => injectq,
				ssra => gnd,
				ssrb => gnd,
				clka => link_clk,
				clkb => clock,
				addra => link_addr_reg,
				addrb => read_addr_reg,
				dob => dp_ram_out);
--		link_dpram: ram16x28d
--		 	port map(
--				wclk				=> link_clk,
--				we					=> '1',
--				addra(3 downto 0)	=> link_addr_reg(3 downto 0),
--				din(27 downto 0)	=> link_data_reg(27 downto 0),
--				addrb( 3 downto 0)	=> read_addr_reg(3 downto 0),
--				dout(27 downto 0)	=> dp_ram_out(27 downto 0)
--			);

		pattern_checker: check_patt
		    port map(
		        clock			=> link_clk,
		        reset			=> reset,
				d(27 downto 0)	=> link_data_reg(27 downto 0),
				error			=> perr
		    );
-- new error checking
process(link_clk)
begin
  if(link_clk'event and link_clk = '1')then
    link_div4(0) <= not link_div4(1);
    link_div4(1) <= link_div4(0);
-- eof must coincide with link_addr equal 6
	 if(link_data_reg(0) = '1' or (link_addr_reg(2) = '1' and link_addr_reg(1) = '1' and link_addr_reg(0) = '0'))then
	   period_err <= link_data_reg(0) xor (link_addr_reg(2) and link_addr_reg(1) and not link_addr_reg(0));
	 end if;
  end if;
end process;
process(clock)
begin
  if(clock'event and clock = '1')then
    clk_sync <= clk_sync(2 downto 0) & link_div4(1);
    clk_div2 <= not clk_div2;
	 if(clk_cntr(3) = '1')then
-- reset after error detected
	   clk_cntr <= "0100";
	 elsif((clk_sync(3) xor clk_sync(2)) /= clk_div2)then
	   if(clk_div2 = '1')then
-- counts up every two system clock
	     clk_cntr <= clk_cntr + 1;
		else
-- counts down every two link clock
	     clk_cntr <= clk_cntr - 1;
		end if;
	 end if;
  end if;
end process;
-- if two clocks' periods are	different, clk_cntr(3) reaches one eventually
clk_err <= clk_cntr(3);
end link_dskew_arch;
