-- title:	top-level backend
-- file:	aa_top.vhd
-- project: cft/cps axial daughterboard (DFEA)
-- device:	U1 backend FPGA
-- target:	Xilinx XCV300-4BG432C
-- author: 	jamieson olsen <jamieson@fnal.gov>
-- created:	28 Nov 2000

-- *** recent revision history ***

-- 08 Sep 2002: 	include *real* turn and crossing counters in the canned L2 record.

-- 07 Oct 2002:		add the ability to completely zero out the CTOC L1/L2 output bus.

-- 16 Oct 2002:		real L2 records are produced.
-- 22 Oct 2002:		fixed bug in cluster translation... now consistant with 2002-01-22a
-- 23 Oct 2002:		add support for fake L2 records.
-- 23 Jan 2003:		change L1 fake "busy" event to "test event"
-- 17 Feb 2003:		fix typo in fake "test" event.
-- 07 Mar 2003:		enhanced communicaiton with the DFE motherboard U49 device.  Supports
--					10-bit (2 sel + 8 data) writes from the motherboard, and returns 8-bits
--					back down to the motherboard -- used to support status pages.  add the ability
-- 					to ignore L1_ACCEPTs.
-- 10 Mar 2003:		extrapolate.vhd now correctly maps track phi values from 0-43, not 1-44
-- 13 Mar 2003:		replace crossing_count module with the new tick_turn module, which is the same one 
-- 					that the L1CTOC and CTTT use.  change around the status bits in the status pages
-- 					DFEA TDR 0.61 reflects these changes.
--  2 Apr 2003:		Isolated track occupancy bit (LSb) set if occupancy is less than 16 doublets hit.
--					16/240 = ~6%.  Fixed BUG in L2 pipeline that was responsible for tick/turn counts
--					being off by 32.
-- 25 Apr 2003:		add tracks and clusters to the fake L2 records.
-- 13 May 2003: 	fix track curvature 1=CCW be consistant throughout DFEA firmware.
--  1 Mar 2004:		cleanup for XST

-- [CFT Tracks]
-- 	muon_trk bus has format VCEePppppp (Max, High, and Medium) or VCEeePppppp (Lowest)
--	same deal for ctoc_trk busses.  muon_trks are 4 clocks earlier than ctoc_trks.
--  start bit marks the first muon_trk.

--
-- [CPS Clusters]
-- 	Two cluster finders share an 18-bit wide bus called cluster[17..0].  See my engineering
--	note 2002-01-22A for timing information and bus signal assignment.  Together, the two 
--	cluster finders deliver up to 8 clusters per crossing.  NOTE: the clusters begin to
--	arrive one clock cycle AFTER the START bit.
--
-- [Control Bits]
-- 	DFE motherboard U18 extracts, delays, and stretches the control bits so that they 
-- 	align to the muon_trk bus:
-- 
-- *** Input Timing revised 8 August 2002 ***
--
--  start       ..../    \____________________________________/    \___________
-- 	muon_trk   	....< T1 >< T2 >< T3 >< T4 >< T5 >< T6 >< T7 ><    ><    >....
-- 	control bit ....<             valid control              ><          
--	ctoc_trk    ....<    ><    ><    ><    >< T1 >< T2 >< T3 >< T4 >< T5 >< T6 >< T7 >....
--  clusters    ....<    >< C1 >< C2 >< C3 >< C4 >_____________...
--  doublets    ....               ...____________< D1 >< D2 >< D3 >< D4 >________...
--	
--	[Motherboard Communication]
--	This FPGA is the only chip on the DFEA that can communicate with the motherboard directly.
--	see eng. note 2002-02-13a (also includes info on Muon test modes)!

library ieee,mylib;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use mylib.dfea_pack.all;
entity aa_top is
	port (

		-- control and diagnostic signals to/from the DFE motherboard...

			clock: 		in  std_logic;		-- 53MHz DFE board clock.
			clock_n:		in  std_logic;		-- inverted 53MHz DFE board clock.
			reset:		in	std_logic;		-- dfe motherboard reset, active low *async*
			r2a:			in	 std_logic;
-- backplane interface signals
         fx:		   in std_logic;
         sync_gap:   in std_logic;
         l1:		   in std_logic;
         cft_reset:  in std_logic;
         u10:		   in std_logic;
         wr_mobo:	   in std_logic;
         rd_mobo:	   in std_logic;
         rd_input:   in std_logic;
         rd_muon:	   in std_logic;
         rd_CTOC:	   in std_logic;
         cap_l1:	   in std_logic;
         cap_l2:	   in std_logic;
		   link_data : in std_logic_vector(111 downto 0);
		   mem_depth : in std_logic_vector(8 downto 0);
		   addr:       in std_logic_vector(9 downto 0);					
		   w_data:     in std_logic_vector(15 downto 0);					
		   r_data:     out std_logic_vector(15 downto 0);					
		-- data from the track finder FPGAs...

			start:			in	std_logic;						-- set to mark first muon_trk
			max_muon_trk:	in	std_logic_vector( 9 downto 0);	-- format VCEePppppp
			high_muon_trk:	in	std_logic_vector( 9 downto 0);	-- format VCEePppppp
			med_muon_trk:	in	std_logic_vector( 9 downto 0);	-- format VCEePppppp
			low_muon_trk:	in	std_logic_vector(10 downto 0);	-- format VCEeePppppp (note 3-bit extended pt)
	
			max_ctoc_trk:	in	std_logic_vector( 9 downto 0);	-- format VCEePppppp
			high_ctoc_trk:	in	std_logic_vector( 9 downto 0);	-- format VCEePppppp
			med_ctoc_trk:	in	std_logic_vector( 9 downto 0);	-- format VCEePppppp
			low_ctoc_trk:	in	std_logic_vector(10 downto 0);	-- format VCEeePppppp (note 3-bit extended pt)
			cphi:  in array_x8y6;
			cluster:		in	std_logic_vector(17 downto 0);	-- format VWwwwCccc VWwwwCccc (A in upper half)
			doublet:		in	std_logic_vector( 2 downto 0);
-- status
			valid_trk:	out	std_logic;	
			bad_fx:		out	std_logic;	
			sg_err:		out	std_logic;	
			locked:		out	std_logic;	

			diag:			out	std_logic_vector(3 downto 0);	-- diagnostic header JP3 pins 9 thru 12
		-- isolation bits to/from DFE backplane

			dbl_trk:	in	std_logic;
			prev_trk:	in	std_logic;						-- used for L1 isolation
			next_trk:	in	std_logic;
			home_trk:	out	std_logic;

		-- outputs to transition board...

			ctoc_out:	out	std_logic_vector(27 downto 0);	-- L1/L2 data to CTOC board
			muon_dat: 	out	std_logic_vector(15 downto 0);	-- list of tracks to Muon L1 trig
			muon_xe:	out	std_logic;						-- XMIT ENABLE control bit for Muon SLDB
			muon_pe:	out	std_logic						-- PARITY ENABLE control bit for Muon SLDB
	);
end aa_top;

architecture aa_top_arch of aa_top is

	-- declare registers (pack 'em into IOBs!)

		signal max_ctoc_trk_reg:	std_logic_vector(10 downto 0);
		signal high_ctoc_trk_reg:	std_logic_vector(10 downto 0);
		signal med_ctoc_trk_reg:	std_logic_vector(10 downto 0);
		signal low_ctoc_trk_reg:	std_logic_vector(10 downto 0);
		signal cluster_reg:			std_logic_vector(17 downto 0);
		signal doublet_reg:			std_logic_vector( 2 downto 0);
		signal start_reg:			std_logic_vector( 5 downto 0);		-- start bit
		signal cr_reg:				std_logic_vector( 4 downto 0);		-- CFT reset
		signal l1acc_reg:			std_logic_vector( 4 downto 0);		-- L1 accept
		signal fx_reg:				std_logic_vector( 4 downto 0);		-- first crossing
		signal sg_reg:				std_logic;							-- sync_gap
		signal mux_seq_reg:			std_logic_vector( 5 downto 0);		-- control the output mux feeding parity

      signal diag_reg:        	std_logic_vector(3 downto 0);

	-- declare signals...

		signal sector: 			std_logic_vector( 3 downto 0);
		signal octant:			std_logic_vector( 2 downto 0);
		signal L1_out:			std_logic_vector(27 downto 0);
		signal L2CFT_out:		std_logic_vector(27 downto 0);
		signal L2CPS_out:		std_logic_vector(27 downto 0);
		signal frame_mux:		std_logic_vector(27 downto 0);
		-- signal l2_xmit:			std_logic;	-- removed 7 May 04
		signal muon_fake:		std_logic;
		signal muon_patt:		std_logic;
		signal ctoc_patt:		std_logic;
		signal ctoc_zero:		std_logic;
		signal ctoc_fake_sel:	std_logic_vector(1 downto 0);
		signal ctoc_L2_fake:	std_logic_vector(1 downto 0);
--		signal mem_depth:		std_logic_vector(6 downto 0);
		signal max_cluster:		std_logic_vector(3 downto 0);	-- format ValidMatch + ClusterIndex used in L2
		signal high_cluster:	std_logic_vector(3 downto 0);
		signal med_cluster:		std_logic_vector(3 downto 0);
		signal low_cluster:		std_logic_vector(3 downto 0);
		signal iso_track:		   std_logic;			-- set if isolated track, used in L2

	-- declare components...		

		component l1_top
			port(
				reset:			in 	std_logic;						-- active low board reset
				clock:			in 	std_logic;						-- 53 MHz
				cr:				in	std_logic;						-- CFT reset (*not* board reset!)
				fx:				in	std_logic;						-- first crossing
				start:			in	std_logic;
				max_track:		in	std_logic_vector(10 downto 0);
				high_track:		in	std_logic_vector(10 downto 0);
				med_track:		in	std_logic_vector(10 downto 0);
				low_track:		in	std_logic_vector(10 downto 0);
				cluster:		in	std_logic_vector(17 downto 0);
				doublet:		in	std_logic_vector( 2 downto 0);
				octant:			in	std_logic_vector( 2 downto 0);	-- octant (0-7)
				sector:			in	std_logic_vector( 3 downto 0);	-- relative sector within a super-sector (0-15)
				max_cluster:	out	std_logic_vector(3 downto 0);	-- format ValidMatch + ClusterIndex
				high_cluster:	out	std_logic_vector(3 downto 0);	-- format ValidMatch + ClusterIndex
				med_cluster:	out	std_logic_vector(3 downto 0);	-- format ValidMatch + ClusterIndex
				low_cluster:	out	std_logic_vector(3 downto 0);	-- format ValidMatch + ClusterIndex
				iso_track:		in	std_logic;
				fake_sel:		in	std_logic_vector( 1 downto 0);	-- select fake L1 outputs to CTOC
				ctoc_out:		out	std_logic_vector(27 downto 0)	-- L1 output to CTOC...
			);
		end component;

		component l2_top
			port(
				clock:			in 	std_logic;
				reset:			in 	std_logic;
				L1acc:			in	std_logic;
				cr:				in	std_logic;
				fx:				in	std_logic;
				octant:			in	std_logic_vector(2 downto 0);
				sector:			in	std_logic_vector(3 downto 0);
				depth:			in	std_logic_vector(8 downto 0);
				send_fake:		in	std_logic_vector(1 downto 0);
				start:			in	std_logic;
				cluster_a:		in	std_logic_vector(8 downto 0);
				cluster_b:		in	std_logic_vector(8 downto 0);
				max_trk:		in	std_logic_vector(10 downto 0);
				high_trk:		in	std_logic_vector(10 downto 0);
				med_trk:		in	std_logic_vector(10 downto 0);
				low_trk:		in	std_logic_vector(10 downto 0);
				max_cluster:	in	std_logic_vector(3 downto 0);
				high_cluster:	in	std_logic_vector(3 downto 0);	
				med_cluster:	in	std_logic_vector(3 downto 0);
				low_cluster:	in	std_logic_vector(3 downto 0);
				iso_track:		in	std_logic;
				pm:				in	std_logic_vector(1 downto 0);
				locked:			out	std_logic;
				bad_fx:			out	std_logic;
				cft_frame:		out	std_logic_vector(27 downto 0);	
				cps_frame:		out	std_logic_vector(27 downto 0)
			);
		end component;

		component parity
			port(
				reset:				in 	std_logic;						-- active low async
				clock:				in 	std_logic;						-- dfe board clock 53mhz
				ctoc_patt:			in	std_logic;
				ctoc_zero:			in	std_logic;
				d:					in	std_logic_vector(27 downto 0);
				q:					out	std_logic_vector(27 downto 0)
			);
		end component;

	COMPONENT mobo_int
	PORT(
		reset : IN std_logic;
		clock : IN std_logic;
		wr_mobo : IN std_logic;
		rd_mobo : IN std_logic;
		addr : IN std_logic_vector(9 downto 0);
		w_data : IN std_logic_vector(15 downto 0);
		fx : IN std_logic;
		sg : IN std_logic;
		start : IN std_logic;          
		r_data : OUT std_logic_vector(15 downto 0);
		sg_err : OUT std_logic;
		new_muon : OUT std_logic;
		muon_fake : OUT std_logic;
		muon_patt : OUT std_logic;
		ctoc_patt : OUT std_logic;
		ctoc_fake_sel : OUT std_logic_vector(1 downto 0);
		ctoc_zero : OUT std_logic;
		ctoc_L2_fake : OUT std_logic_vector(1 downto 0);
--		mem_depth : OUT std_logic_vector(6 downto 0);
		rel_sect : OUT std_logic_vector(3 downto 0);
		octant : OUT std_logic_vector(2 downto 0)
		);
	END COMPONENT;

		component muon_top
			port(
				clock:		in	std_logic;
				sg:			in	std_logic;						-- sync gap, active high; see comments above
				cr:			in	std_logic;						-- CFT system reset (not the same as board power on reset!)
				reset:		in	std_logic;						-- active low board reset!
				start:		in	std_logic;						-- set to mark first track
				max_trk: 	in 	std_logic_vector(9 downto 0);	-- tracks from Pt bins format VCEePpppppp
				high_trk: 	in 	std_logic_vector(9 downto 0);	-- tracks from Pt bins format VCEePpppppp
				med_trk: 	in 	std_logic_vector(9 downto 0);	-- tracks from Pt bins format VCEePpppppp
				low_trk: 	in 	std_logic_vector(10 downto 0);	-- tracks from Pt bins format VCEeePpppppp
				send_fake:	in	std_logic;
				send_patt:	in	std_logic;
				dat: 		out std_logic_vector(15 downto 0);	-- data to muon L1
				val:		out	std_logic;						-- copy of dat(15)
				xe:			out	std_logic;						-- control signal to muon SLDB
				pe:			out std_logic						-- control signal to muon SLDB
		    );
		end component;
	COMPONENT muon_match
	PORT(
		clock : IN std_logic;
	 	r2a : in std_logic;
	 	new_muon : in std_logic;
	 	muon_patt : in std_logic;
	 	word5 : in std_logic;
	 	iso_trk : in std_logic;
		track : IN std_logic_vector(15 downto 0);
		cphi : IN array_x8y6;          
		muon_data : OUT std_logic_vector(15 downto 0)
		);
	END COMPONENT;
signal new_muon : std_logic;
signal single_trk : std_logic;
signal iso_trk : std_logic;
signal home_trk_i : std_logic;
signal addr_q : std_logic_vector(9 downto 0);
signal busy : std_logic;
signal wr_muon : std_logic;
signal wr_l1 : std_logic;
signal start_CPS : std_logic;
signal last_CFT : std_logic;
signal wr_CPS : std_logic;
signal wr_CFT : std_logic;
signal link_din : std_logic_vector(127 downto 0);
signal link_dl : std_logic_vector(127 downto 0);
signal link_dout : std_logic_vector(63 downto 0);
signal start_n : std_logic;
signal wr_input : std_logic;
signal link_offset : std_logic_vector(8 downto 0);
signal link_wa : std_logic_vector(8 downto 0);
signal link_ra : std_logic_vector(8 downto 0);
signal input_wa : std_logic_vector(8 downto 0);
signal input_ra : std_logic_vector(9 downto 0);
signal mu_offset : std_logic_vector(8 downto 0);
signal mu_dl_wa : std_logic_vector(9 downto 0);
signal mu_dl_ra : std_logic_vector(9 downto 0);
signal l1_offset : std_logic_vector(8 downto 0);
signal l1_wa : std_logic_vector(8 downto 0);
signal l1_ra : std_logic_vector(8 downto 0);
signal l1_din : std_logic_vector(31 downto 0);
signal l1_dl : std_logic_vector(31 downto 0);
signal muon_data : std_logic_vector(15 downto 0);
signal muon_trk : std_logic_vector(15 downto 0);
signal muon_din : std_logic_vector(15 downto 0);
signal muon_dl : std_logic_vector(15 downto 0);
signal muon_dout : std_logic_vector(15 downto 0);
signal muon_wa : std_logic_vector(9 downto 0);
signal muon_ra : std_logic_vector(9 downto 0);
signal wr_l2 : std_logic;
signal L2_din : std_logic_vector(31 downto 0);
signal CTOC_dout : std_logic_vector(31 downto 0);
signal CTOC_wa : std_logic_vector(8 downto 0);
signal CTOC_ra : std_logic_vector(8 downto 0);
signal L1_cntr : std_logic_vector(2 downto 0);
signal L1_out_q : std_logic_vector(3 downto 0);
signal CFT_out_q : std_logic_vector(2 downto 0);
signal L1_header : std_logic;
signal CFT_header : std_logic;
signal cap_l2_dl : std_logic;
signal ctoc_out_i : std_logic_vector(27 downto 0);
signal ctoc_out_q : std_logic_vector(27 downto 0);
signal invalid_l1 : std_logic;
attribute IOB: string;
attribute IOB of muon_dat : signal is "false";
begin
	-- combinatorial stuff...
		diag(3 downto 0)		<= diag_reg(3 downto 0);
-- 7/8/05 renamed to allow use inside the module
		ctoc_out <= ctoc_out_i;
		-- control the mux that chooses which data bus (L1_out, L2CFT_out, or L2CPS_out)
		-- to pass on to the parity module.

			frame_mux(27 downto 0) <= L2CFT_out(27 downto 0) when (mux_seq_reg(5 downto 0)<"011010") else
									  L2CPS_out(27 downto 0) when (mux_seq_reg(5 downto 0)>"100011" and mux_seq_reg(5 downto 0)<"101110") else
									  L1_out(27 downto 0)    when (mux_seq_reg(5 downto 0)>"110000") else
									  X"0000000";

-- input registers removed to reduce latency	Aug-09-04
--				max_ctoc_trk_reg(10 downto 0)	<= max_ctoc_trk(9 downto 8) & '0' & max_ctoc_trk(7 downto 0);
--				high_ctoc_trk_reg(10 downto 0)	<= high_ctoc_trk(9 downto 8)  & '0' & high_ctoc_trk(7 downto 0);
--				med_ctoc_trk_reg(10 downto 0)	<= med_ctoc_trk(9 downto 8) & '0' & med_ctoc_trk(7 downto 0);
--				low_ctoc_trk_reg(10 downto 0)	<= low_ctoc_trk(10 downto 0);
				doublet_reg(2 downto 0)			<= doublet(2 downto 0);
				cluster_reg(17 downto 0)		<= cluster(17 downto 0);
	-- sequential stuff...

		-- register the start bit, ctoc_track busses, cluster bus, doublets and
		-- control bits (CR, FX).  Since the start bit and the control bits come 
		-- in early, delay all of them by 4 clock cycles.

--		register_inputs: process(clock, reset) change to sync reset Aug-06-04
	  register_inputs: process(clock)
	  begin
		 if rising_edge(clock) then
			if (reset='0') then
-- input registers removed to reduce latency	Aug-09-04
				max_ctoc_trk_reg(10 downto 0)	<= "00000000000";
				high_ctoc_trk_reg(10 downto 0)	<= "00000000000";
				med_ctoc_trk_reg(10 downto 0)	<= "00000000000";
				low_ctoc_trk_reg(10 downto 0)	<= "00000000000";
--				doublet_reg(2 downto 0)			<= "000";
--				cluster_reg(17 downto 0)		<= "000000000000000000";
				diag_reg(3 downto 0)				<= "0000";
				cr_reg(4 downto 0)				<= "00000";
				fx_reg(4 downto 0)				<= "00000";
				l1acc_reg(4 downto 0)			<= "00000";
				start_reg(5 downto 0)			<= "000000";
				L1_out_q								<= "0000";
				CFT_out_q							<= "000";
				L1_header							<= '0';
				CFT_header							<= '0';
				sg_reg 								<= '0';
				mux_seq_reg(5 downto 0)			<= "111111";
			else
				max_ctoc_trk_reg(10 downto 0)	<= max_ctoc_trk(9 downto 8) & '0' & max_ctoc_trk(7 downto 0);
				high_ctoc_trk_reg(10 downto 0)	<= high_ctoc_trk(9 downto 8)  & '0' & high_ctoc_trk(7 downto 0);
				med_ctoc_trk_reg(10 downto 0)	<= med_ctoc_trk(9 downto 8) & '0' & med_ctoc_trk(7 downto 0);
				low_ctoc_trk_reg(10 downto 0)	<= low_ctoc_trk(10 downto 0);
--				doublet_reg(2 downto 0)			<= doublet(2 downto 0);
--				cluster_reg(17 downto 0)		<= cluster(17 downto 0);
-- diag_reg(0) is muon start signal
            diag_reg(0)                <= start;
-- diag_reg(1) is muon valid signal
				diag_reg(1)						<= muon_data(15);
-- diag_reg(2) is L1 header with fx bit set
				L1_out_q <= L1_out(27 downto 25) & L1_out(17);
				if(L1_out_q = "1111") then
				  L1_header	<= '1';
				else
				  L1_header	<= '0';
				end if;
				diag_reg(2)						<= L1_header;
-- diag_reg(3) is L2CFT header
				CFT_out_q <= L2CFT_out(27 downto 25);
				if(CFT_out_q = "111" and mux_seq_reg(5) = '0') then
				  CFT_header	<= '1';
				else
				  CFT_header	<= '0';
				end if;
				diag_reg(3) <= CFT_header;
				cr_reg(4 downto 1)		<= cr_reg(3 downto 0);
				cr_reg(0)				<= cft_reset;

				fx_reg(4 downto 1)		<= fx_reg(3 downto 0);
				fx_reg(0)				<= fx;

				l1acc_reg(4 downto 1)	<= l1acc_reg(3 downto 0);
				l1acc_reg(0)			<= l1;

				start_reg(5 downto 1)	<= start_reg(4 downto 0);
				start_reg(0)			<= start;

				sg_reg 					<= sync_gap;
			
				-- control the sequencer that determines when to send L1 or L2 data
				-- if ctoc_L2_fake="01" then ignore L1_ACCEPT and stay in L1 mode all the time.

-- all input registers for tracks, doublets and clusters have been removed, start, cr and fx delay
-- must be reduced by one everywhere  Aug-09-04
--				if ((l1acc_reg(3)='1') and (l1acc_reg(4)='0') and (ctoc_L2_fake(1 downto 0) /= "01") ) then
				if ((l1acc_reg(2)='1') and (l1acc_reg(3)='0') and (ctoc_L2_fake(1 downto 0) /= "01") ) then
					mux_seq_reg(5 downto 0) <= "000000";
				elsif (mux_seq_reg(5 downto 0) /= "111111") then
					mux_seq_reg(5 downto 0) <= mux_seq_reg(5 downto 0) + 1;
				end if;

			 end if;
		  end if;
		end process;

	-- instantiate components...

		-- This module communicates with the outside world via the DFEM U49 CPLD.  The 
		-- DFEC can write several bytes to U49, and it serializes them and sends them
		-- to this module, where they are converted back to parallel.  L2 depth, bus control,
		-- sector number, etc. are some of the parameters that must be set after initialization.


		-- Refer to my engineering note 2002-02-07A and 2002-02-04a for more info.

		motherboard_interface: mobo_int
			port map(
		reset => reset,
		clock => clock,
		wr_mobo => wr_mobo,
		rd_mobo => rd_mobo,
		addr => addr,
		w_data => w_data,
		r_data => r_data,
		fx => fx_reg(0),
		sg => sg_reg,
		start => start_reg(0),
		sg_err => sg_err,
		new_muon => new_muon,
		muon_fake => muon_fake,
		muon_patt => muon_patt,
		ctoc_patt => ctoc_patt,
		ctoc_fake_sel => ctoc_fake_sel,
		ctoc_zero => ctoc_zero,
		ctoc_L2_fake => ctoc_l2_fake,
--		mem_depth => mem_depth,
		rel_sect => sector,
		octant => octant
	);

		-- generates L1 CTOC records.  

		level_one_top: l1_top
			port map(
				reset					=> reset,
				clock					=> clock,
-- all input registers for tracks, doublets and clusters have been removed, start, cr and fx delay
-- must be reduced by one everywhere  Aug-09-04
--				cr						=> cr_reg(4),
--				fx						=> fx_reg(4),
--				start					=> start_reg(4),
				cr						=> cr_reg(3),
				fx						=> fx_reg(3),
				start					=> start_reg(3),
				max_track(10 downto 0)	=> max_ctoc_trk_reg(10 downto 0),
				high_track(10 downto 0)	=> high_ctoc_trk_reg(10 downto 0),
				med_track(10 downto 0)	=> med_ctoc_trk_reg(10 downto 0),
				low_track(10 downto 0)	=> low_ctoc_trk_reg(10 downto 0),
				cluster(17 downto 0)	=> cluster_reg(17 downto 0),
				doublet(2 downto 0)		=> doublet_reg(2 downto 0),
				octant( 2 downto 0)		=> octant(2 downto 0),
				sector( 3 downto 0)		=> sector(3 downto 0),
				fake_sel(1 downto 0)	=> ctoc_fake_sel(1 downto 0),
				ctoc_out(27 downto 0)	=> L1_out(27 downto 0),
				iso_track 				=> iso_track,

				max_cluster(3 downto 0)  => max_cluster(3 downto 0),
				high_cluster(3 downto 0) => high_cluster(3 downto 0),
				med_cluster(3 downto 0)  => med_cluster(3 downto 0),
				low_cluster(3 downto 0)  => low_cluster(3 downto 0)
			);


		-- Pass-n-Mark bits (pm) not implemented.  zeroed out.

		level_two_top: l2_top
			port map(
				clock					=> clock,
				reset 					=> reset,

				octant(2 downto 0)		=> octant(2 downto 0),
				sector(3 downto 0)		=> sector(3 downto 0),
				depth		=> mem_depth,
				send_fake(1 downto 0)	=> ctoc_L2_fake(1 downto 0),

-- all input registers for tracks, doublets and clusters have been removed, start, cr and l1_acc delay
-- must be reduced by one everywhere  Aug-09-04
--				start					=> start_reg(1),
--				L1acc					=> l1acc_reg(0),
--				cr						=> cr_reg(1),
--				fx						=> fx_reg(1),
				start					=> start_reg(0),
				L1acc					=> l1,
				cr						=> cr_reg(0),
				fx						=> fx_reg(0),

				cluster_a(8 downto 0)	=> cluster_reg(17 downto 9),	-- VWwwwCccc
				cluster_b(8 downto 0)	=> cluster_reg( 8 downto 0),	-- VWwwwCccc

				max_trk(10 downto 0)	=> max_ctoc_trk_reg(10 downto 0),
				high_trk(10 downto 0)	=> high_ctoc_trk_reg(10 downto 0),
				med_trk(10 downto 0)	=> med_ctoc_trk_reg(10 downto 0),
				low_trk(10 downto 0)	=> low_ctoc_trk_reg(10 downto 0),

				max_cluster(3 downto 0)		=> max_cluster(3 downto 0),
				high_cluster(3 downto 0)	=> high_cluster(3 downto 0),	
				med_cluster(3 downto 0)		=> med_cluster(3 downto 0),
				low_cluster(3 downto 0)		=> low_cluster(3 downto 0),

				iso_track					=> iso_track,
				pm(1 downto 0)				=> "00",
				locked						=> locked,
				bad_fx						=> bad_fx,
				cft_frame(27 downto 0)		=> L2CFT_out(27 downto 0),
				cps_frame(27 downto 0)		=> L2CPS_out(27 downto 0)
			);


		-- parity generator does the horizontal and vertical parity for the 
		-- data going to the CTOC board.  if CTOC_PATT is high then it drives
		-- a johnson counter on the CTOC_OUT bus.

		parity_gen: parity
			port map(
				reset			=> 	reset,
				clock			=> 	clock,
				ctoc_patt		=> 	ctoc_patt,
				ctoc_zero		=> 	ctoc_zero,
				d(27 downto 0)	=>	frame_mux(27 downto 0),
-- 7/8/05 renamed to allow use inside the module
--				q(27 downto 0)	=>	ctoc_out(27 downto 0)
				q(27 downto 0)	=>	ctoc_out_i
			);

		-- chooses the six highest Pt tracks and sends them to MUON L1.
		-- this module takes care of the SLDB protocol and it responds to
		-- SYNC_GAP and CFT_RESET.  Signals are registered internally...
		-- connect directly to I/O pins with no registers in between.

		muon_track_sorter: muon_top
			port map(
				clock					=>	clock,
				sg						=>	sync_gap,
				cr						=>	cft_reset,
				reset					=>	reset,
-- muon tracks' input registers were inside muon_top module, start signal timing
-- changes are inside muon_top module	 Aug-13-04
				start					=>  start_reg(5),
				max_trk(9 downto 0)	=>	max_muon_trk(9 downto 0),
				high_trk(9 downto 0)	=>	high_muon_trk(9 downto 0),
				med_trk(9 downto 0)	=>	med_muon_trk(9 downto 0),
				low_trk(10 downto 0)	=>	low_muon_trk(10 downto 0),
				send_fake				=> 	muon_fake,
				send_patt				=> 	muon_patt,
				dat(15 downto 0)		=>	muon_trk,
				val						=>  valid_trk,
				xe						=>	muon_xe,
				pe						=>	muon_pe
		    );
home_trk <= home_trk_i;
process(clock)
begin
  if(clock'event and clock = '1')then
	 if(start_reg(0) = '1')then
	   home_trk_i <= muon_data(15);
	 end if;
	 if(start_reg(1) = '1')then
	   single_trk <= home_trk_i and not muon_data(15);
	 end if;
	 if(start_reg(4) = '1')then
	   iso_trk <= single_trk and not prev_trk and not next_trk and not dbl_trk;
	 end if;
	 if(start_reg(2) = '1')then
	   iso_track <= iso_trk;
	 end if;
--	 muon_data13 <= start_reg(4) and iso_trk and not r2a;
  end if;
end process;
I_muon_match: muon_match PORT MAP(
		clock => clock,
		r2a => r2a,
		new_muon => new_muon,
		muon_patt => muon_patt,
		word5 => start_reg(5),
		iso_trk => iso_trk,
		track => muon_trk,
		cphi => cphi,
		muon_data => muon_data
	);
-- input link data delay
g_link_dl: for i in 0 to 3 generate
link_din(i*32+31 downto i*32) <= "0000" & link_data(i*28+27 downto i*28);
		i_link_dl: RAMB16_S36_S36
		   port map(
				dia => gnd32,
			   dib => link_din(i*32+31 downto i*32),
				dipa => gnd4,
				dipb => gnd4,
				ena => vcc,
				enb => vcc,
				wea => gnd,
				web => vcc,
				ssra => gnd,
				ssrb => gnd,
				clka => clock_n,
				clkb => clock_n,
				addra => link_ra,
				addrb => link_wa,
				doa => link_dl(i*32+31 downto i*32));
end generate;
process(clock_n)
begin
  if(clock_n'event and clock_n = '1')then
    start_n <= start;
	 link_offset <= mem_depth + 20;
	 link_wa <= link_ra + link_offset;
	 link_ra <= link_ra + 1;
  end if;
end process;
-- input link data capture
g_input: for i in 0 to 3 generate
		input_dpram: RAMB16_S18_S36
		   port map(
				dia => gnd16,
			   dib => link_dl(i*32+31 downto i*32),
				dipa => gnd2,
				dipb => gnd4,
				ena => vcc,
				enb => vcc,
				wea => gnd,
				web => wr_input,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock_n,
				addra => input_ra,
				addrb => input_wa,
				doa => link_dout(i*16+15 downto i*16));
end generate;
process(clock_n)
begin
  if(clock_n'event and clock_n = '1')then
    if(start_n = '1')then
      wr_input <= cap_l1;
	 end if;
	 if(wr_input = '0')then
	   input_wa(2 downto 0) <= "000";
	 else
	   input_wa(2 downto 0) <= input_wa(2 downto 0) + 1;
	 end if;
  end if;
end process;
process(clock)
begin
  if(clock'event and clock = '1')then
	 addr_q <= addr;
  end if;
end process;
input_wa(8 downto 3) <= "000000";
input_ra <= "000000" & addr(3 downto 0);
r_data <= link_dout(15 downto 0) when rd_input = '1' and addr_q(5 downto 4) = "00" else (others => 'Z');
r_data <= link_dout(31 downto 16) when rd_input = '1' and addr_q(5 downto 4) = "01" else (others => 'Z');
r_data <= link_dout(47 downto 32) when rd_input = '1' and addr_q(5 downto 4) = "10" else (others => 'Z');
r_data <= link_dout(63 downto 48) when rd_input = '1' and addr_q(5 downto 4) = "11" else (others => 'Z');
-- output data capture
		i_muon_dl: RAMB16_S18_S18
		   port map(
				dia => gnd16,
			   dib => muon_din,
				dipa => gnd2,
				dipb => gnd2,
				ena => vcc,
				enb => vcc,
				wea => gnd,
				web => vcc,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock,
				addra => mu_dl_ra,
				addrb => mu_dl_wa,
				doa => muon_dl);
		i_l1_dl: RAMB16_S36_S36
		   port map(
				dia => gnd32,
			   dib => l1_din,
				dipa => gnd4,
				dipb => gnd4,
				ena => vcc,
				enb => vcc,
				wea => gnd,
				web => vcc,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock,
				addra => l1_ra,
				addrb => l1_wa,
				doa => l1_dl);
l1_din <= invalid_l1 & "000" & ctoc_out_q;
i_wr_l1: srlc16e port map(
            d => cap_l1,
				ce =>	start,
				clk => clock,
				a0 => gnd,
				a1 => gnd,
				a2 => vcc,
				a3 => gnd,
				q => wr_l1);
mu_dl_wa(9) <= '0';
mu_dl_ra(9) <= '0';
process(clock)
begin
  if(clock'event and clock = '1')then
    if(mux_seq_reg(5) = '0' and mux_seq_reg(1) = '1')then
	   invalid_l1 <= '1';
	 elsif(mux_seq_reg(5 downto 4) = "11" and mux_seq_reg(1 downto 0) = "11")then
	   invalid_l1 <= '0';
	 end if;
    ctoc_out_q <= ctoc_out_i;
    mu_offset <= mem_depth + 11;
    mu_dl_wa(8 downto 0) <= mu_dl_ra(8 downto 0) + mu_offset;
    mu_dl_ra(8 downto 0) <= mu_dl_ra(8 downto 0) + 1;
    l1_offset <= mem_depth + 21;
    l1_wa <= l1_ra + l1_offset;
    l1_ra <= l1_ra + 1;
  end if;
end process;
		muon_dpram: RAMB16_S18_S18
		   port map(
				dia => gnd16,
			   dib => muon_dl,
				dipa => gnd2,
				dipb => gnd2,
				ena => vcc,
				enb => vcc,
				wea => gnd,
				web => wr_muon,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock,
				addra => muon_ra,
				addrb => muon_wa,
				doa => muon_dout);
muon_ra <= "0000000" & addr(2 downto 0);
muon_wa(9 downto 3) <= "0000000";
r_data <= muon_dout when rd_muon = '1' else (others => 'Z');
-- this delay is introduced to account for the gap between L2CFT and L2CPS data
i_start_CPS: srlc16 port map(
            d => last_CFT,
				clk => clock,
				a0 => gnd,
				a1 => gnd,
				a2 => gnd,
				a3 => vcc,
				q => start_CPS);
process(clock)
begin
  if(clock'event and clock = '1')then
    wr_muon <= cap_l1;
    if(reset = '0')then
	   muon_dat <= (others => '0');
      muon_din <= (others => '0');
	 else
	   muon_dat <= muon_data;
      muon_din <= muon_data;
	 end if;
    if(wr_muon = '0')then
	   muon_wa(2 downto 0) <= "000";
	 else
	   muon_wa(2 downto 0) <= muon_wa(2 downto 0) + 1;
	 end if;
	 cap_l2_dl <= cap_l2;
	 if(cap_l2_dl = '1' and l1acc_reg(4) = '1')then
	   wr_CFT <= '1';
	 elsif(CTOC_wa(4 downto 3) = "11" and CTOC_wa(0) = '1')then
	   wr_CFT <= '0';
	 end if;
	 last_CFT <= CTOC_wa(6)and CTOC_wa(4)and CTOC_wa(3);
	 if(start_CPS = '1')then
	   wr_CPS <= '1';
	 elsif(CTOC_wa(3) = '1' and CTOC_wa(0) = '1')then
	   wr_CPS <= '0';
	 end if;
	 wr_l2 <= wr_CPS or wr_CFT;
	 CTOC_wa(6) <= wr_CFT;
	 CTOC_wa(5) <= (wr_CFT and u10) or wr_CPS;
	 if(wr_l2 = '1')then
	   CTOC_wa(4 downto 0) <= CTOC_wa(4 downto 0) + 1;
	 else
	   CTOC_wa(4 downto 0) <= (wr_CPS and u10) & "0000";
	 end if;
	 if(wr_l1 = '0')then
	   l1_cntr <= "000";
	 else
	   l1_cntr <= l1_cntr + 1;
	 end if;
  end if;
end process;
		CTOC_dpram: RAMB16_S36_S36
		   port map(
				dia => l1_dl,
			   dib => L2_din,
				dipa => gnd4,
				dipb => gnd4,
				ena => vcc,
				enb => vcc,
				wea => wr_l1,
				web => wr_l2,
				ssra => gnd,
				ssrb => gnd,
				clka => clock,
				clkb => clock,
				addra => CTOC_ra,
				addrb => CTOC_wa,
				doa => CTOC_dout);
L2_din(31 downto 28) <= "0000";
L2_din(27 downto 0) <= ctoc_out_q;
CTOC_ra <= "00" & addr(7 downto 1) when wr_l1 = '0' else "00001" & u10 & l1_cntr;
CTOC_wa(8 downto 7) <= "00";
r_data <= CTOC_dout(31 downto 16) when rd_CTOC = '1' and addr_q(0) = '1' else (others => 'Z');
r_data <= CTOC_dout(15 downto 0) when rd_CTOC = '1' and addr_q(0) = '0' else (others => 'Z');
process(clock)
begin
  if(clock'event and clock = '1')then
    if(wr_muon = '0')then
	   muon_wa(2 downto 0) <= "000";
	 else
	   muon_wa(2 downto 0) <= muon_wa(2 downto 0) + 1;
	 end if;
  end if;
end process;
end aa_top_arch;
