-- L1_top.vhd
-- jamieson olsen <jamieson@fnal.gov>

--	level one processor top level design to tie together all of the 
--	L1 modules in the DFEA/U1 design.  This design assumes that all 
--	inputs are REGISTERED externally!

--	reference my engineering note 2002-02-12A

-- *** revision history *** 
--
-- 12 Feb 2002:	  	includes L1 MUON sender, isolation, doublet counting, L1 output
--					formatting, and sum of Pt.  Does NOT include parity generation.  
--					L2 hooks not included yet.  Pass-N-Mark bit are not supported --
--					always uses "00" for those bits.  Isolated track occupancy bit is
--					always zero.

-- 15 May 2002: 	delay the doublet count even more, to be consistant with
--					eng note 2002-02-08a.  blank out the rest of the isolation
--					bits whenever the iso_v bit is NOT set.  Set L1 PM bits to "11"
--					during the SYNC_GAPs.

-- 25 Jun 2002:		Set L1 PM bits to "11" for First Crossing.

-- 08 Jul 2002: 	P&M bits are not implemented.  Include FX and CR control bits in
--					CTOC L1 header.  updated the L1_FORMATTER to be consistant with 
--					latest protocols.  Pt bin enumeration is 00=max, 11=low, etc.

-- 12 Aug 2002:		remove the L1muon logic, adjust delays on tracks, start bit, control bits,
--					and clusters.  doublet count shifted in two bits at a time.  change width of
--					track busses.  Control bits and tracks must be aligned with the Start bit.
--					clusters lead the start bit by three ticks (start marks the LAST cluster)
--					doublets require four clocks to shift in, and they lag the first track by
--					one tick.  When computing the difference between projected CFT tracks and CPS
--					centroids, use 4 bits.  

-- 29 Aug 2002:		add multiple fake events to the L1 CTOC output, controlled by mobo_int.vhd.

-- 16 Oct 2002:		add isolated track stub for passing to L2_top logic.

-- 21 Oct 2002:		add reset to cluster_match modules.

--  2 Apr 2003:		isolated track "golden track" occupancy bit set if occupancy is less than
--					16 doublets in the home sector.  "iso_oc"

--  7 Apr 2003:		fix bug with the isolated track occupancy bit.
-- 13 May 2003:		consistant curve def 1=CCW
--  7 May 2004:		tighten the cluster/track matching windows to 1,2,2,4


library ieee;
use ieee.std_logic_1164.all;

entity l1_top is
	port(
		reset:				in 	std_logic;						-- active low board reset
		clock:				in 	std_logic;						-- 53 MHz

		cr:					in	std_logic;						-- CFT reset, aligned with start bit
		fx:					in	std_logic;						-- first crossing, aligned with start bit
		start:				in	std_logic;						-- set to mark first track, aligned with start bit

		max_track:			in	std_logic_vector(10 downto 0);	-- track format VCEeePppppp
		high_track:			in	std_logic_vector(10 downto 0);	-- track format VCEeePppppp
		med_track:			in	std_logic_vector(10 downto 0);	-- track format VCEeePppppp
		low_track:			in	std_logic_vector(10 downto 0);	-- track format VCEeePppppp
		cluster:			in	std_logic_vector(17 downto 0);	-- clusters lead tracks by 4 clocks
		doublet:			in	std_logic_vector( 2 downto 0);	-- doublets shifted in two bits at a time

		octant:				in	std_logic_vector( 2 downto 0);	-- octant (0-7)
		sector:				in	std_logic_vector( 3 downto 0);	-- relative sector within a super-sector (0-15)
	
		max_cluster:		out	std_logic_vector(3 downto 0);	-- format ValidMatch + ClusterIndex(2..0)
		high_cluster:		out	std_logic_vector(3 downto 0);	
		med_cluster:		out	std_logic_vector(3 downto 0);	
		low_cluster:		out	std_logic_vector(3 downto 0);

		iso_track:			in	std_logic;

		fake_sel:			in	std_logic_vector(1 downto 0);
		ctoc_out:			out	std_logic_vector(27 downto 0)	-- L1 output to CTOC...
	);
end l1_top;

architecture l1_top_arch of l1_top is

	-- define signals...

		signal 	max_pt:			std_logic_vector(1 downto 0);
		signal 	high_pt:		std_logic_vector(1 downto 0);
		signal 	med_pt:			std_logic_vector(1 downto 0);
		signal 	low_pt:			std_logic_vector(1 downto 0);

		signal	max_window:		std_logic_vector(3 downto 0);
		signal	high_window:	std_logic_vector(3 downto 0);
		signal	med_window:		std_logic_vector(3 downto 0);
		signal	low_window:		std_logic_vector(3 downto 0);

		signal 	cphi0:			std_logic_vector(5 downto 0);
		signal 	cphi1:			std_logic_vector(5 downto 0);
		signal 	cphi2:			std_logic_vector(5 downto 0);
		signal 	cphi3:			std_logic_vector(5 downto 0);
		signal 	cphi4:			std_logic_vector(5 downto 0);
		signal 	cphi5:			std_logic_vector(5 downto 0);
		signal 	cphi6:			std_logic_vector(5 downto 0);
		signal 	cphi7:			std_logic_vector(5 downto 0);

		signal 	max_start:		std_logic;
		signal 	high_start:		std_logic;
		signal 	med_start:		std_logic;
		signal 	low_start:		std_logic;

		signal 	max_curve:		std_logic;
		signal 	high_curve:		std_logic;
		signal 	med_curve:		std_logic;
		signal 	low_curve:		std_logic;

		signal 	max_proj_track:		std_logic_vector(5 downto 0);
		signal 	high_proj_track:	std_logic_vector(5 downto 0);
		signal 	med_proj_track:		std_logic_vector(5 downto 0);
		signal 	low_proj_track:		std_logic_vector(5 downto 0);

		signal 	max_cnt_valid:		std_logic;
		signal	max_cnt_cw:			std_logic_vector(2 downto 0);
		signal	max_cnt_ccw:		std_logic_vector(2 downto 0);
		signal	max_cnt_ncw:		std_logic_vector(2 downto 0);
		signal	max_cnt_nccw:		std_logic_vector(2 downto 0);

		signal	high_cnt_cw:		std_logic_vector(2 downto 0);
		signal	high_cnt_ccw:		std_logic_vector(2 downto 0);
		signal	high_cnt_ncw:		std_logic_vector(2 downto 0);
		signal	high_cnt_nccw:		std_logic_vector(2 downto 0);

		signal	med_cnt_cw:			std_logic_vector(2 downto 0);
		signal	med_cnt_ccw:		std_logic_vector(2 downto 0);
		signal	med_cnt_ncw:		std_logic_vector(2 downto 0);
		signal	med_cnt_nccw:		std_logic_vector(2 downto 0);

		signal	low_cnt_cw:			std_logic_vector(2 downto 0);
		signal	low_cnt_ccw:		std_logic_vector(2 downto 0);
		signal	low_cnt_ncw:		std_logic_vector(2 downto 0);
		signal	low_cnt_nccw:		std_logic_vector(2 downto 0);

		signal 	cluster_count:		std_logic_vector(1 downto 0);
		signal 	doublets_dly:		std_logic_vector(7 downto 0);
		signal 	fx_dly:				std_logic;
		signal 	cr_dly:				std_logic;
		signal 	total_pt:			std_logic_vector(7 downto 0);


		signal	pm:						std_logic_vector(1 downto 0);
		signal 	valid_cluster_array:	std_logic_vector(7 downto 0);

		signal 	iso_assoc:			std_logic;						-- set if iso track matches to a cluster...
		signal 	iso_curv:			std_logic;						-- set if isolated track curves CCW...
		signal 	iso_oc:				std_logic;						-- "golden isolated track"
		signal 	iso_pt:				std_logic_vector(1 downto 0);	-- pt bin of the isolated track 
		signal 	max_iso:			std_logic;
		signal 	high_iso:			std_logic;
		signal 	med_iso:			std_logic;

	-- define registers...

	-- declare components...

		component extrapolate
			port(
				reset:		in 	std_logic;							-- active low async
				clock:		in 	std_logic;							-- dfe board clock 53mhz
				start_in:	in	std_logic;							-- set to mark the first track
				track:		in	std_logic_vector(10 downto 0);		-- format VCEeePppppp
				pt_bin:		in	std_logic_vector( 1 downto 0);		-- 00=max, ...  11=low
				proj:		out	std_logic_vector( 4 downto 0);		-- projected track phi
				valid:		out	std_logic;							-- set if track is valid
				curve:		out	std_logic;							-- set if track curves CCW (decreasing phi)
				start_out:	out	std_logic							-- set to mark the first track
			);
		end component;

		component cluster_unpacker
			port(
				clock:		in 	std_logic;						-- dfe board clock 53mhz
				reset:		in	std_logic;						-- active low, async.
				cluster_a:	in	std_logic_vector(8 downto 0);	-- output from cluster finder "a"
				cluster_b:	in	std_logic_vector(8 downto 0);	-- output from cluster finder "b"
				start:		in	std_logic;						-- set to indicate the *LAST* cluster
				cphi0:		out	std_logic_vector(5 downto 0);	-- v+phi, send to the cluster finders...
				cphi1:		out	std_logic_vector(5 downto 0);
				cphi2:		out	std_logic_vector(5 downto 0);
				cphi3:		out	std_logic_vector(5 downto 0);
				cphi4:		out	std_logic_vector(5 downto 0);
				cphi5:		out	std_logic_vector(5 downto 0);
				cphi6:		out	std_logic_vector(5 downto 0);
				cphi7:		out	std_logic_vector(5 downto 0)
			);
		end component;

		component cluster_match
			port (
				clock:			in 	std_logic;
				reset:			in	std_logic;
				track:			in	std_logic_vector(5 downto 0);	-- valid + projected track phi
				start:			in	std_logic;						-- set when 1st track is clocked in
				curve:			in	std_logic;						-- CFT track curvature bit, set if CCW.
				window:			in	std_logic_vector(3 downto 0);
				cphi0:			in	std_logic_vector(5 downto 0);	-- cluster 0
				cphi1:			in	std_logic_vector(5 downto 0);	-- cluster 1
				cphi2:			in	std_logic_vector(5 downto 0);	-- cluster 2 , etc.
				cphi3:			in	std_logic_vector(5 downto 0);
				cphi4:			in	std_logic_vector(5 downto 0);
				cphi5:			in	std_logic_vector(5 downto 0);
				cphi6:			in	std_logic_vector(5 downto 0);
				cphi7:			in	std_logic_vector(5 downto 0);
				cnt_cw:			out	std_logic_vector(2 downto 0);	-- count of CW  curving tracks w/  cluster match
				cnt_ccw:		out	std_logic_vector(2 downto 0);	-- count of CCW curving tracks w/  cluster match
				cnt_ncw:		out	std_logic_vector(2 downto 0);	-- count of CW  curving tracks w/o cluster match
				cnt_nccw:		out	std_logic_vector(2 downto 0);	-- count of CCW curving tracks w/o cluster match
				cnt_valid:		out	std_logic;						-- set when the counts are valid
				cluster_index:	out	std_logic_vector(2 downto 0);
				valid_match:	out	std_logic
			);
		end component;

		component sum_pt
			port(
				reset:			in 	std_logic;
				clock:			in 	std_logic;
				start:			in	std_logic;		
				max_ext_pt:		in	std_logic_vector(2 downto 0);
				high_ext_pt:	in	std_logic_vector(2 downto 0);
				med_ext_pt:		in	std_logic_vector(2 downto 0);
				low_ext_pt:		in	std_logic_vector(2 downto 0);
				max_valid:		in	std_logic;
				high_valid:		in	std_logic;
				med_valid:		in	std_logic;
				low_valid:		in	std_logic;
				total_pt:		out	std_logic_vector(7 downto 0)
			);
		end component;

		component L1_delay
			port(
				reset:			in 	std_logic;
				clock:			in 	std_logic;
				fx:				in	std_logic;
				cr:				in	std_logic;
				doublets:		in	std_logic_vector(2 downto 0);
				fx_dly:			out	std_logic;
				cr_dly:			out	std_logic;
				doublets_dly:	out	std_logic_vector(7 downto 0)
			);
		end component;

		component level_one_format
			port(
				clock:		in	std_logic;						-- 53 MHz
				reset:		in	std_logic;						-- active low
				start:		in	std_logic;						-- active high, set once every 7 cycles.
				pm:			in	std_logic_vector(1 downto 0);	-- pass-n-mark field
				fx:			in	std_logic;						-- FIRST_XING
				cr:			in	std_logic;						-- CFT_RESET (aka INIT)
				oct:		in	std_logic_vector(2 downto 0);	-- octant field (0-7)
				rel_sect:	in	std_logic_vector(3 downto 0);	-- relative sector within a super-sector (0-15)
				max_cw:		in	std_logic_vector(2 downto 0);	-- count of CW  curving max tracks w/  cluster match
				max_ccw:	in	std_logic_vector(2 downto 0);	-- count of CCW curving max tracks w/  cluster match
				max_ncw:	in	std_logic_vector(2 downto 0);	-- count of CW  curving max tracks w/o cluster match
				max_nccw:	in	std_logic_vector(2 downto 0);	-- count of CCW curving max tracks w/o cluster match
				high_cw:	in	std_logic_vector(2 downto 0);	-- count of CW  curving high tracks w/  cluster match
				high_ccw:	in	std_logic_vector(2 downto 0);	-- count of CCW curving high tracks w/  cluster match
				high_ncw:	in	std_logic_vector(2 downto 0);	-- count of CW  curving high tracks w/o cluster match
				high_nccw:	in	std_logic_vector(2 downto 0);	-- count of CCW curving high tracks w/o cluster match
				med_cw:		in	std_logic_vector(2 downto 0);	-- count of CW  curving med tracks w/  cluster match
				med_ccw:	in	std_logic_vector(2 downto 0);	-- count of CCW curving med tracks w/  cluster match
				med_ncw:	in	std_logic_vector(2 downto 0);	-- count of CW  curving med tracks w/o cluster match
				med_nccw:	in	std_logic_vector(2 downto 0);	-- count of CCW curving med tracks w/o cluster match
				low_cw:		in	std_logic_vector(2 downto 0);	-- count of CW  curving low tracks w/  cluster match
				low_ccw:	in	std_logic_vector(2 downto 0);	-- count of CCW curving low tracks w/  cluster match
				low_ncw:	in	std_logic_vector(2 downto 0);	-- count of CW  curving low tracks w/o cluster match
				low_nccw:	in	std_logic_vector(2 downto 0);	-- count of CCW curving low tracks w/o cluster match
				clusters:	in	std_logic_vector(1 downto 0);	-- number of clusters found in the home sector
				doublets:	in	std_logic_vector(7 downto 0);	-- number of doublets found in the home sector
				sum_pt:		in	std_logic_vector(5 downto 0);	-- sum the Pt of all found tracks 
				iso_v:		in	std_logic;						-- set if a valid isolated track exists
				iso_assoc:	in	std_logic;						-- set if the isolated track matches to a cluster
				iso_curv:	in	std_logic;						-- set if iso track curvature is CCW
				iso_pt:		in	std_logic_vector(1 downto 0);	-- Pt bin of the iso track ('00'=max, .. '11'=low)
				iso_oc:		in	std_logic;						-- isolated occupancy bit?  leave zero.
				fake_sel:	in	std_logic_vector(1 downto 0);	-- select different fake events or real data
				ctoc_out:	out	std_logic_vector(27 downto 0)	-- level_one formatted output.	
			);	
		end component;

		component cluster_counter
			port(
				reset:		in	std_logic;						-- active low, async.
				clock:		in	std_logic;						-- 53MHz
				val_clust:	in	std_logic_vector(7 downto 0);	-- cluster valid bits
				c_cnt:		out	std_logic_vector(1 downto 0)	-- passed on to L1 formatter
			);	
		end component;

 begin

	-- combinatorial stuff...

		-- pass-n-mark bits are not implemented...

			pm(1 downto 0)	<= "00";

		-- assign the Pt bin markers
		-- see engineering note 2001-12-31A for details...

			max_pt(1 downto 0)	<= "00";
			high_pt(1 downto 0)	<= "01";
			med_pt(1 downto 0)	<= "10";
			low_pt(1 downto 0)	<= "11";

		-- here is where the track-cluster matching window is defined.
		-- see my engineering note 2002-01-31A for details...

			-- max_window(3 downto 0)	<=	"0010";
			-- high_window(3 downto 0)	<=	"0100";
			-- med_window(3 downto 0)	<=	"0110";
			-- low_window(3 downto 0)	<=	"1000";

			max_window(3 downto 0)	<=	"0001";		-- new, added 7 May 2004
			high_window(3 downto 0)	<=	"0010";
			med_window(3 downto 0)	<=	"0010";
			low_window(3 downto 0)	<=	"0100";


		-- concatenate all of the valid cluster bits into a 1x8 array for the
		-- cluster counter module...

			valid_cluster_array(7 downto 0)	<=	cphi0(5) & cphi1(5) & cphi2(5) & cphi3(5) &
												cphi4(5) & cphi5(5) & cphi6(5) & cphi7(5);

		-- isolated track "occupancy bit" turned on when occupancy is less than 16 
		-- doublets hit.  ~6%

			iso_oc <= '1' when (doublets_dly(7 downto 4)="0000") else '0';

		-- if iso_track is set then look at the track counts to determine which
		-- pt bin the isolated track came from.  If iso_track is set, only *ONE*
		-- of the counts should be non-zero (by definition).

			max_iso		<=	'1' when (max_cnt_cw(2 downto 0)   /= "000") else
							'1' when (max_cnt_ccw(2 downto 0)  /= "000") else
							'1' when (max_cnt_ncw(2 downto 0)  /= "000") else
							'1' when (max_cnt_nccw(2 downto 0) /= "000") else
							'0';

			high_iso	<=	'1' when (high_cnt_cw(2 downto 0)   /= "000") else
							'1' when (high_cnt_ccw(2 downto 0)  /= "000") else
							'1' when (high_cnt_ncw(2 downto 0)  /= "000") else
							'1' when (high_cnt_nccw(2 downto 0) /= "000") else
							'0';

			med_iso		<=	'1' when (med_cnt_cw(2 downto 0)   /= "000") else
							'1' when (med_cnt_ccw(2 downto 0)  /= "000") else
							'1' when (med_cnt_ncw(2 downto 0)  /= "000") else
							'1' when (med_cnt_nccw(2 downto 0) /= "000") else
							'0';

			-- isolation Pt bin 00=max, 11=low...

			iso_pt(1 downto 0)	<=	"00" when (max_iso='1') else
									"01" when (max_iso='0' and high_iso='1') else
									"10" when (max_iso='0' and high_iso='0' and med_iso='1') else
									"11";

		-- if iso_track is set then look at the track counts to determine
		-- the curvature of the isolated track.  set if CCW curvature.

			iso_curv <=		max_cnt_ccw(2)   or max_cnt_ccw(1)   or max_cnt_ccw(0) or
							max_cnt_nccw(2)  or max_cnt_nccw(1)  or max_cnt_nccw(0) or
							high_cnt_ccw(2)  or high_cnt_ccw(1)  or high_cnt_ccw(0) or
							high_cnt_nccw(2) or high_cnt_nccw(1) or high_cnt_nccw(0) or
							med_cnt_ccw(2)   or med_cnt_ccw(1)   or med_cnt_ccw(0) or
							med_cnt_nccw(2)  or med_cnt_nccw(1)  or med_cnt_nccw(0) or
							low_cnt_ccw(2)   or low_cnt_ccw(1)   or low_cnt_ccw(0) or
							low_cnt_nccw(2)  or low_cnt_nccw(1)  or low_cnt_nccw(0);


		-- if iso_track is set then look at the track counts to determine if
		-- if the isolated track matched to a cluster or not.  set iso_assoc if
		-- there's a match...

			iso_assoc <= 	max_cnt_cw(2) or max_cnt_cw(1) or max_cnt_cw(0) or
						 	max_cnt_ccw(2) or max_cnt_ccw(1) or max_cnt_ccw(0) or
							high_cnt_cw(2) or high_cnt_cw(1) or high_cnt_cw(0) or
						 	high_cnt_ccw(2) or high_cnt_ccw(1) or high_cnt_ccw(0) or
							med_cnt_cw(2) or med_cnt_cw(1) or med_cnt_cw(0) or
						 	med_cnt_ccw(2) or med_cnt_ccw(1) or med_cnt_ccw(0) or
							low_cnt_cw(2) or low_cnt_cw(1) or low_cnt_cw(0) or
							low_cnt_ccw(2) or low_cnt_ccw(1) or low_cnt_ccw(0);

	-- sequential stuff...

	-- instantiate components...


		-- flattens clusters and preps them for the cluster finder modules.

		cps_cluster_unpacker: cluster_unpacker
			port map(
				clock					=> 	clock,
				reset					=>	reset,
				cluster_a(8 downto 0)	=>	cluster(17 downto 9),
				cluster_b(8 downto 0)	=>	cluster(8  downto 0),
				start					=>	start,					-- set to mark the LAST cluster
				cphi0(5 downto 0)		=>	cphi0(5 downto 0),
				cphi1(5 downto 0)		=>	cphi1(5 downto 0),
				cphi2(5 downto 0)		=>	cphi2(5 downto 0),
				cphi3(5 downto 0)		=>	cphi3(5 downto 0),
				cphi4(5 downto 0)		=>	cphi4(5 downto 0),
				cphi5(5 downto 0)		=>	cphi5(5 downto 0),
				cphi6(5 downto 0)		=>	cphi6(5 downto 0),
				cphi7(5 downto 0)		=>	cphi7(5 downto 0)
			);

		-- count the number of valid clusters.  this module delays the final 
		-- sum long enough so that it's valid by the the time the cluster
		-- finder asserts the CNT_VALID output.

		cluster_cntr: cluster_counter
			port map(
				reset					=>	reset,
				clock					=>	clock,
				val_clust(7 downto 0)	=>	valid_cluster_array(7 downto 0),	
				c_cnt(1 downto 0)		=>	cluster_count(1 downto 0)
			);	

		-- projects a valid CFT track up onto the CPS layer.  
		-- this function looks at the Phi and Pt bin of each 
		-- track.

		max_extrap: extrapolate
			port map(
				reset				=> 	reset,
				clock				=>	clock,
				start_in			=> 	start,
				track(10 downto 0)	=> 	max_track(10 downto 0),
				pt_bin(1 downto 0)	=>	max_pt(1 downto 0),
				proj(4 downto 0)	=>	max_proj_track(4 downto 0),
				valid				=>	max_proj_track(5),
				curve				=>	max_curve,
				start_out			=>	max_start
			);

		high_extrap: extrapolate
			port map(
				reset				=> 	reset,
				clock				=>	clock,
				start_in			=> 	start,
				track(10 downto 0)	=> 	high_track(10 downto 0),
				pt_bin(1 downto 0)	=>	high_pt(1 downto 0),
				proj(4 downto 0)	=>	high_proj_track(4 downto 0),
				valid				=>	high_proj_track(5),
				curve				=>	high_curve,
				start_out			=>	high_start
			);

		med_extrap: extrapolate
			port map(
				reset				=> 	reset,
				clock				=>	clock,
				start_in			=> 	start,
				track(10 downto 0)	=> 	med_track(10 downto 0),
				pt_bin(1 downto 0)	=>	med_pt(1 downto 0),
				proj(4 downto 0)	=>	med_proj_track(4 downto 0),
				valid				=>	med_proj_track(5),
				curve				=>	med_curve,
				start_out			=>	med_start
			);

		low_extrap: extrapolate
			port map(
				reset				=> 	reset,
				clock				=>	clock,
				start_in			=> 	start,
				track(10 downto 0)	=> 	low_track(10 downto 0),
				pt_bin(1 downto 0)	=>	low_pt(1 downto 0),
				proj(4 downto 0)	=>	low_proj_track(4 downto 0),
				valid				=>	low_proj_track(5),
				curve				=>	low_curve,
				start_out			=>	low_start
			);

		-- cluster_match loads all of the clusters in parallel while
		-- accepting tracks from a track finder serially.  Fairly long
		-- latency to determine the counts for L1.  L2 stubs already
		-- in place...


		max_cluster_match: cluster_match
			port map(
				clock						=>	clock,
				reset						=> 	reset,
				track						=>	max_proj_track(5 downto 0),
				start						=>	max_start,
				curve						=>	max_curve,
				window(3 downto 0)			=>	max_window(3 downto 0),
				cphi0(5 downto 0)			=>	cphi0(5 downto 0),
				cphi1(5 downto 0)			=>	cphi1(5 downto 0),
				cphi2(5 downto 0)			=>	cphi2(5 downto 0),
				cphi3(5 downto 0)			=>	cphi3(5 downto 0),
				cphi4(5 downto 0)			=>	cphi4(5 downto 0),
				cphi5(5 downto 0)			=>	cphi5(5 downto 0),
				cphi6(5 downto 0)			=>	cphi6(5 downto 0),
				cphi7(5 downto 0)			=>	cphi7(5 downto 0),
				cnt_cw(2 downto 0)			=>	max_cnt_cw(2 downto 0),
				cnt_ccw(2 downto 0)			=>	max_cnt_ccw(2 downto 0),
				cnt_ncw(2 downto 0)			=>	max_cnt_ncw(2 downto 0),
				cnt_nccw(2 downto 0)		=>	max_cnt_nccw(2 downto 0),
				cnt_valid					=>	max_cnt_valid,
				cluster_index(2 downto 0)	=>	max_cluster(2 downto 0),
				valid_match					=>	max_cluster(3)
			);

		high_cluster_match: cluster_match
			port map(
				clock						=>	clock,
				reset						=> 	reset,
				track						=>	high_proj_track(5 downto 0),
				start						=>	high_start,
				curve						=>	high_curve,
				window(3 downto 0)			=>	high_window(3 downto 0),
				cphi0(5 downto 0)			=>	cphi0(5 downto 0),
				cphi1(5 downto 0)			=>	cphi1(5 downto 0),
				cphi2(5 downto 0)			=>	cphi2(5 downto 0),
				cphi3(5 downto 0)			=>	cphi3(5 downto 0),
				cphi4(5 downto 0)			=>	cphi4(5 downto 0),
				cphi5(5 downto 0)			=>	cphi5(5 downto 0),
				cphi6(5 downto 0)			=>	cphi6(5 downto 0),
				cphi7(5 downto 0)			=>	cphi7(5 downto 0),
				cnt_cw(2 downto 0)			=>	high_cnt_cw(2 downto 0),
				cnt_ccw(2 downto 0)			=>	high_cnt_ccw(2 downto 0),
				cnt_ncw(2 downto 0)			=>	high_cnt_ncw(2 downto 0),
				cnt_nccw(2 downto 0)		=>	high_cnt_nccw(2 downto 0),
				cnt_valid					=>	open,
				cluster_index(2 downto 0)	=>	high_cluster(2 downto 0),
				valid_match					=>	high_cluster(3)
			);

		med_cluster_match: cluster_match
			port map(
				clock						=>	clock,
				reset						=> 	reset,
				track						=>	med_proj_track(5 downto 0),
				start						=>	med_start,
				curve						=>	med_curve,
				window(3 downto 0)			=>	med_window(3 downto 0),
				cphi0(5 downto 0)			=>	cphi0(5 downto 0),
				cphi1(5 downto 0)			=>	cphi1(5 downto 0),
				cphi2(5 downto 0)			=>	cphi2(5 downto 0),
				cphi3(5 downto 0)			=>	cphi3(5 downto 0),
				cphi4(5 downto 0)			=>	cphi4(5 downto 0),
				cphi5(5 downto 0)			=>	cphi5(5 downto 0),
				cphi6(5 downto 0)			=>	cphi6(5 downto 0),
				cphi7(5 downto 0)			=>	cphi7(5 downto 0),
				cnt_cw(2 downto 0)			=>	med_cnt_cw(2 downto 0),
				cnt_ccw(2 downto 0)			=>	med_cnt_ccw(2 downto 0),
				cnt_ncw(2 downto 0)			=>	med_cnt_ncw(2 downto 0),
				cnt_nccw(2 downto 0)		=>	med_cnt_nccw(2 downto 0),
				cnt_valid					=>	open,
				cluster_index(2 downto 0)	=>	med_cluster(2 downto 0),
				valid_match					=>	med_cluster(3)
			);

		low_cluster_match: cluster_match
			port map(
				clock						=>	clock,
				reset						=> 	reset,
				track						=>	low_proj_track(5 downto 0),
				start						=>	low_start,
				curve						=>	low_curve,
				window(3 downto 0)			=>	low_window(3 downto 0),
				cphi0(5 downto 0)			=>	cphi0(5 downto 0),
				cphi1(5 downto 0)			=>	cphi1(5 downto 0),
				cphi2(5 downto 0)			=>	cphi2(5 downto 0),
				cphi3(5 downto 0)			=>	cphi3(5 downto 0),
				cphi4(5 downto 0)			=>	cphi4(5 downto 0),
				cphi5(5 downto 0)			=>	cphi5(5 downto 0),
				cphi6(5 downto 0)			=>	cphi6(5 downto 0),
				cphi7(5 downto 0)			=>	cphi7(5 downto 0),
				cnt_cw(2 downto 0)			=>	low_cnt_cw(2 downto 0),
				cnt_ccw(2 downto 0)			=>	low_cnt_ccw(2 downto 0),
				cnt_ncw(2 downto 0)			=>	low_cnt_ncw(2 downto 0),
				cnt_nccw(2 downto 0)		=>	low_cnt_nccw(2 downto 0),
				cnt_valid					=>	open,
				cluster_index(2 downto 0)	=>	low_cluster(2 downto 0),
				valid_match					=>	low_cluster(3)
			);

		-- looks at all the valid tracks as they come in.  For each valid
		-- track the extended Pt and Pt are fed into a look up table and 
		-- the average GeV is calculated.  Total_PT output is valid by the time
		-- the cluster finder CNT_VALID is asserted.  NOTE: range is 0 to 255 
		--	GeV.  This module does not look for a "rollover" condition:
		-- 	that is, 258 GeV will show up as 2 GeV.

		sum_of_pt: sum_pt
			port map(
				reset					=> 	reset,
				clock					=>	clock,
				start					=>	start,	
				max_ext_pt(2 downto 0)	=>	max_track(8 downto 6),
				high_ext_pt(2 downto 0)	=>	high_track(8 downto 6),
				med_ext_pt(2 downto 0)	=>	med_track(8 downto 6),
				low_ext_pt(2 downto 0)	=>	low_track(8 downto 6),
				max_valid				=>	max_track(10),
				high_valid				=>	high_track(10),
				med_valid				=>	med_track(10),
				low_valid				=>	low_track(10),
				total_pt(7 downto 0)	=>	total_pt(7 downto 0)
			);

		-- The MEDIUM track finder adds up the number of doublets
		-- and passes it on to this module.  Basically just delays it
		-- long enough so that it's valid when the cluster finder 
		-- CNT_VALID is asserted.  FIRST_XING and CFT_RESET are also
		-- delayed so that they can be inserted into the L1 CTOC output.

		L1_delayer: L1_delay
			port map(
				reset					=> 	reset,
				clock					=>	clock,
				fx						=>  fx,
				cr						=> 	cr,
				doublets(2 downto 0)	=>	doublet(2 downto 0),

				fx_dly						=> 	fx_dly,
				cr_dly						=> 	cr_dly,
				doublets_dly(7 downto 0)	=>	doublets_dly(7 downto 0)
			);

		l1_formatter: level_one_format
			port map(
				clock					=>	clock,
				reset					=>	reset,
				start					=>	max_cnt_valid,

				pm(1 downto 0)			=>	pm(1 downto 0),
				fx						=> 	fx_dly,
				cr						=> 	cr_dly,

				oct(2 downto 0)			=>	octant(2 downto 0),
				rel_sect(3 downto 0)	=>	sector(3 downto 0),

				max_cw(2 downto 0)		=>	max_cnt_cw(2 downto 0),
				max_ccw(2 downto 0)		=>	max_cnt_ccw(2 downto 0),
				max_ncw(2 downto 0)		=>	max_cnt_ncw(2 downto 0),
				max_nccw(2 downto 0)	=>	max_cnt_nccw(2 downto 0),

				high_cw(2 downto 0)		=>	high_cnt_cw(2 downto 0),
				high_ccw(2 downto 0)	=>	high_cnt_ccw(2 downto 0),
				high_ncw(2 downto 0)	=>	high_cnt_ncw(2 downto 0),
				high_nccw(2 downto 0)	=>	high_cnt_nccw(2 downto 0),

				med_cw(2 downto 0)		=>	med_cnt_cw(2 downto 0),
				med_ccw(2 downto 0)		=>	med_cnt_ccw(2 downto 0),
				med_ncw(2 downto 0)		=>	med_cnt_ncw(2 downto 0),
				med_nccw(2 downto 0)	=>	med_cnt_nccw(2 downto 0),

				low_cw(2 downto 0)		=>	low_cnt_cw(2 downto 0),
				low_ccw(2 downto 0)		=>	low_cnt_ccw(2 downto 0),
				low_ncw(2 downto 0)		=>	low_cnt_ncw(2 downto 0),
				low_nccw(2 downto 0)	=>	low_cnt_nccw(2 downto 0),

				clusters(1 downto 0)	=>	cluster_count(1 downto 0),
				doublets(7 downto 0)	=>	doublets_dly(7 downto 0),
				sum_pt(5 downto 0)		=>	total_pt(7 downto 2),

				iso_v					=>	iso_track,
				iso_assoc				=>	iso_assoc,
				iso_curv				=>	iso_curv,
				iso_pt(1 downto 0)		=>	iso_pt(1 downto 0),
				iso_oc					=>	iso_oc,

				fake_sel(1 downto 0)	=> 	fake_sel(1 downto 0),
				ctoc_out(27 downto 0)	=>	ctoc_out(27 downto 0)
			);	

end l1_top_arch;


