-- L2_clustsort.vhd
-- jamieson olsen <jamieson@fnal.gov>
-- 27 Sept 2002

-- cluster (and best matching track data) is clocked into this device IN ORDER OF 
-- INCREASING PHI, although gaps may be present.  this module compresses the gaps out
-- and also decides which clusters will be reported in the L2CPS record.

-- clusters are stored IF:

-- 1. cluster must be valid
-- 2. centroid must be in the home sector (greater than 7, less than 24)

-- start is set when the first cluster/track is clocked in.  When start is seen
-- the SEQ_REG counter is cleared.  After that, SEQ_REG counts up to 1111 and stops.
-- FIFO writes are allowed on counts 0000-0111, and from 1000-1111 FIFO reads are
-- enabled.  Once the FIFO goes empty, the outputs of this module will zero out.

-- see engineering note 2002-09-23a for details.
-- 13 May 2003:  consistant curve 1 = CCW

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L2_clustsort is
	port(
		clock:		in	std_logic;
		reset:		in	std_logic;

		start:		in	std_logic;		-- set to mark first data coming in

		tvalid:		in	std_logic;						-- valid track
		curve:		in	std_logic;		
		cm:			in	std_logic;						-- cluster match
		pt:			in	std_logic_vector(1 downto 0);
		extpt:		in	std_logic_vector(2 downto 0);	-- extended pt
		cvalid:		in	std_logic;	
		centroid:	in	std_logic_vector(4 downto 0);	
		width:		in	std_logic_vector(2 downto 0);

		cluster_count:	out	std_logic_vector(3 downto 0);	-- number of clusters to be reported (0-8)
		frame_sel:		out	std_logic_vector(1 downto 0);	-- tell L2CPS_FORMAT what frames to send.
		
		tvalid_out:		out	std_logic;						-- valid track
		curve_out:		out	std_logic;		
		cm_out:			out	std_logic;						-- cluster match
		pt_out:			out	std_logic_vector(1 downto 0);
		extpt_out:		out	std_logic_vector(2 downto 0);	-- extended pt
		cvalid_out:		out	std_logic;						-- set if the cluster is valid
		centroid_out:	out	std_logic_vector(4 downto 0);	-- cluster centroid	
		width_out:		out	std_logic_vector(2 downto 0)	-- width of the cluster
    );
end L2_clustsort;


architecture L2_clustsort_arch of L2_clustsort is

	-- define registers...
	
		signal din_reg:		std_logic_vector(16 downto 0);
		signal wptr_reg:	std_logic_vector( 3 downto 0);
		signal rptr_reg:	std_logic_vector( 3 downto 0);
		signal seq_reg:		std_logic_vector( 4 downto 0);
		signal ccnt_reg:	std_logic_vector( 3 downto 0);

	-- define signals

		signal FIFO_write:	std_logic;
		signal FIFO_read:	std_logic;
		signal FIFO_empty:	std_logic;
		signal keep:		std_logic;

		signal dout:		std_logic_vector(16 downto 0);

	-- declare components...

		component ram17x16d 
		 	port (
				wclk:		in	std_logic;						-- sync write clk
				we:			in	std_logic;						-- active high
				addra:		in	std_logic_vector( 3 downto 0);	-- write port address
				din:		in	std_logic_vector(16 downto 0);
				addrb:		in	std_logic_vector( 3 downto 0);	-- read port address
				dout:		out	std_logic_vector(16 downto 0)	-- read data
			);
		end component;
		
 begin

	-- combinatorial stuff...

		-- if the cluster is valid and the centroid is in the home sector, keep it for readout 
		-- in the L2CPS record.

			keep <= '1' when (din_reg(8 downto 4)>"00111" and din_reg(8 downto 4)<"11000" and din_reg(9)='1') else
					'0';

		-- FIFO controls and flags

			FIFO_empty <= '1' when (rptr_reg(3 downto 0) = wptr_reg(3 downto 0)) else
						  '0';
	
			FIFO_write <= '1' when (keep='1' and seq_reg(4 downto 3)="00") else '0';
	
			FIFO_read <= '1' when (seq_reg(4 downto 0)>"01001") else '0';

		-- output the number of words loaded into the FIFO. (this is reset when start is asserted)

			cluster_count(3 downto 0) <= ccnt_reg(3 downto 0);

		-- tell the L2CPS formatter module what type of frames to send...

			frame_sel(1 downto 0) <= "01" when (seq_reg(4 downto 0)="01000") else
									 "10" when (seq_reg(4 downto 0)="01001") else
									 "11" when (seq_reg(4 downto 0)>"01001" and FIFO_empty='0') else		-- send data as long as FIFO is not empty
									 "00";

		-- assign the outputs

			tvalid_out 				 <= dout(16);
			curve_out  				 <= dout(15);
			pt_out(1 downto 0) 		 <= dout(14 downto 13);
			extpt_out(2 downto 0) 	 <= dout(12 downto 10);
			cvalid_out 				 <= dout(9);
			centroid_out(4 downto 0) <= dout(8 downto 4);
			width_out(2 downto 0) 	 <= dout(3 downto 1);
	  		cm_out 				  	 <= dout(0);

	-- sequential stuff...

--		regs: process(clock, reset)  change to sync reset Aug-06-04
		regs: process(clock)
		 begin
			if rising_edge(clock) then
			 if (reset='0') then
				din_reg(16 downto 0) <= "00000000000000000";
				wptr_reg(3 downto 0) <= "0000";
				rptr_reg(3 downto 0) <= "0000";
				seq_reg(4 downto 0)  <= "11111";
				ccnt_reg(3 downto 0) <= "1111";
			 else
				din_reg(16) 			<= tvalid;
				din_reg(15) 			<= curve;
				din_reg(14 downto 13) 	<= pt(1 downto 0);
				din_reg(12 downto 10) 	<= extpt(2 downto 0);
				din_reg(9) 				<= cvalid;
				din_reg(8 downto 4) 	<= centroid(4 downto 0);
				din_reg(3 downto 1) 	<= width(2 downto 0);
				din_reg(0) 				<= cm;

				if (start='1') then
					wptr_reg(3 downto 0) <= "0000";	
					ccnt_reg(3 downto 0) <= "0000";		-- cluster count
				elsif (FIFO_write='1') then
					wptr_reg(3 downto 0) <= wptr_reg(3 downto 0) + 1;
					ccnt_reg(3 downto 0) <= ccnt_reg(3 downto 0) + 1;
				end if;

				if (start='1') then
					rptr_reg(3 downto 0) <= "0000";			
				elsif (FIFO_read='1' and FIFO_empty='0') then
					rptr_reg(3 downto 0) <= rptr_reg(3 downto 0) + 1;
				end if;

				if (start='1') then
					seq_reg(4 downto 0) <= "00000";
				elsif (seq_reg(4 downto 0) /= "11111") then
					seq_reg(4 downto 0) <= seq_reg(4 downto 0) + 1;
				end if;


			 end if;
		  end if;
		end process;

	-- instantiate components...

		csort_ram: ram17x16d 
		 	port map(
				wclk			  => clock,
				we				  => FIFO_write,
				addra(3 downto 0) => wptr_reg(3 downto 0),	-- write pointer
				din(16 downto 0)  => din_reg(16 downto 0),
				addrb(3 downto 0) => rptr_reg(3 downto 0),	-- read pointer
				dout(16 downto 0) => dout(16 downto 0)
			);

		
end L2_clustsort_arch;

