-- L2_packer.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- accept L1 data from multiple sources, all skewed in time relative to each other.  
-- delay and align them and then output them as a 80x7 block of bits to be 
-- passed onto the L2_pipeline.  See engineering note 2002-03-22A for a picture of 
-- what this 80x7 block is supposed to look like.

-- this module must calculate the number of valid tracks in each Pt bin.  range is 0-6
-- per bin.

-- ISO, PM, Turn and Crossing are aligned in time one tick AFTER the match data.

-- *** revision history ***
-- 15 October 2002: created, functionally simulated.  See engineering note 2002-10-15a
-- 22 October 2002:	fix bug in cluster translation.
-- 13 March   2003: NOTE: turn and crossing data arrives one clock cycle later, but this is OK.
--					no logic changes to this file.



library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

--library virtex;

entity L2_packer is
	port(
		clock:				in	std_logic;	  	-- 53MHz
		reset:				in	std_logic;	  	-- active low, async.
		start:				in	std_logic;		-- set to mark the first clusters clocked in

		-- raw cluster information (4 deep)

		cluster_a:			in	std_logic_vector(8 downto 0);	-- clusters 0-3 format VWwwwCccc
		cluster_b:			in	std_logic_vector(8 downto 0);	-- clusters 4-7 format VWwwwCccc
	
		-- misc. stuff.

		crossing:			in	std_logic_vector(7 downto 0);	-- crossing counter
		turn:				in	std_logic_vector(15 downto 0);	-- turn counter
		pm:					in	std_logic_vector(1 downto 0);	-- pass-n-mark
		iso_track:			in	std_logic;						-- isolated track

		-- CFT tracks (6 deep, total of 24 tracks)

		max_trk:			in	std_logic_vector(10 downto 0);	-- format VCEeePppppp
		high_trk:			in	std_logic_vector(10 downto 0);	-- format VCEeePppppp
		med_trk:			in	std_logic_vector(10 downto 0);	-- format VCEeePppppp
		low_trk:			in	std_logic_vector(10 downto 0);	-- format VCEeePppppp

		-- tells which tracks are matched to which cluster number.

		max_cluster:		in	std_logic_vector(3 downto 0);	-- format cluster_match + cluster_number(2..0)
		high_cluster:		in	std_logic_vector(3 downto 0);	-- format cluster_match + cluster_number(2..0)
		med_cluster:		in	std_logic_vector(3 downto 0);	-- format cluster_match + cluster_number(2..0)
		low_cluster:		in	std_logic_vector(3 downto 0);	-- format cluster_match + cluster_number(2..0)


		bor:				out	std_logic;						-- set to mark first frame out of 7
		dout:				out	std_logic_vector(79 downto 0)	-- aligned data going into pipeline
	);
end L2_packer;

architecture L2_packer_arch of L2_packer is

	-- declare registers...

		signal max_count:	std_logic_vector(2 downto 0);	-- number of max pt tracks
		signal high_count:	std_logic_vector(2 downto 0);	-- number of high pt tracks
		signal med_count:	std_logic_vector(2 downto 0);	-- number of med pt tracks
		signal low_count:	std_logic_vector(2 downto 0);	-- number of low pt tracks

		signal max_cnt_reg:		std_logic_vector(2 downto 0);
		signal high_cnt_reg:	std_logic_vector(2 downto 0);
		signal med_cnt_reg:		std_logic_vector(2 downto 0);
		signal low_cnt_reg:		std_logic_vector(2 downto 0);

		signal frame_count:	std_logic_vector(2 downto 0);
		signal out_reg:		std_logic_vector(79 downto 0);
		signal bor_reg: 	std_logic;

	-- declare signals...

		signal new_cluster_a:	std_logic_vector(8 downto 0);	-- VCccccWww
		signal new_cluster_b:	std_logic_vector(8 downto 0);	-- VCccccWww

		signal tdelay:	std_logic_vector(3 downto 0);
		signal cdelay: 	std_logic_vector(3 downto 0);
		signal mdelay:	std_logic_vector(3 downto 0);

		signal match_data:	  	std_logic_vector(15 downto 0);	-- format max(cm,cnum(2..0)) 

		signal match_data_d:  	std_logic_vector(15 downto 0);
		signal max_trk_d:		std_logic_vector(10 downto 0);
		signal high_trk_d:		std_logic_vector(10 downto 0);
		signal med_trk_d:		std_logic_vector(10 downto 0);
		signal low_trk_d:		std_logic_vector(10 downto 0);
		signal cluster_a_d:		std_logic_vector(8 downto 0);
		signal cluster_b_d:		std_logic_vector(8 downto 0);

		signal frame_mux:		std_logic_vector(79 downto 0);
		signal frame0:			std_logic_vector(79 downto 0);
		signal frame13:			std_logic_vector(79 downto 0);
		signal frame46:			std_logic_vector(79 downto 0);

	-- declare components...

		component SRL9x16e
		 	port (
				clock:		in	std_logic;						
				ce:			in	std_logic;						-- active high chip enable
				d:			in	std_logic_vector(8 downto 0);	-- data input
				depth:		in	std_logic_vector(3 downto 0);	-- 1 to 16 clocks deep
				q:			out	std_logic_vector(8 downto 0)	-- input delay, delayed by (depth+1) clock ticks
			);
		end component;

		component SRL11x16e
		 	port (
				clock:		in	std_logic;						
				ce:			in	std_logic;						-- active high chip enable
				d:			in	std_logic_vector(10 downto 0);	-- data input
				depth:		in	std_logic_vector(3 downto 0);	-- 1 to 16 clocks deep
				q:			out	std_logic_vector(10 downto 0)	-- input delay, delayed by (depth+1) clock ticks
			);
		end component;

		component SRL16x16e
		 	port (
				clock:		in	std_logic;						
				ce:			in	std_logic;						-- active high chip enable
				d:			in	std_logic_vector(15 downto 0);	-- data input
				depth:		in	std_logic_vector(3 downto 0);	-- 1 to 16 clocks deep
				q:			out	std_logic_vector(15 downto 0)	-- input delay, delayed by (depth+1) clock ticks
			);
		end component;


begin

	-- combinatorial stuff...

		match_data(15 downto 0) <= max_cluster(3 downto 0) & high_cluster(3 downto 0) & med_cluster(3 downto 0) & low_cluster(3 downto 0);

		bor 			  <= bor_reg;
		dout(79 downto 0) <= out_reg(79 downto 0);


		-- translate the cluster format from VWwwwCccc to VCccccWww.  If clusters are wider than 7, 
		-- set the width field to 111.  Add 16 to cluster A's centroid.  See engineering note 
		-- 2002-01-22a for more info.

			new_cluster_a(8 downto 3) <= cluster_a(8) & '1' & cluster_a(3 downto 0);
			new_cluster_a(2 downto 0) <= cluster_a(6 downto 4) when (cluster_a(7)='0') else "111";

			new_cluster_b(8 downto 3) <= cluster_b(8) & '0' & cluster_b(3 downto 0);
			new_cluster_b(2 downto 0) <= cluster_b(6 downto 4) when (cluster_b(7)='0') else "111";


		-- specify the shift register delays to make the data line up properly.
			tdelay(3 downto 0) <= "1000";
			cdelay(3 downto 0) <= "1010";
			mdelay(3 downto 0) <= "0010";

		-- define how fields are packed into frames...

			-- first frame

				frame0(79 downto 0) <=	max_count(2 downto 0) & high_count(2 downto 0) & med_count(2 downto 0) & 
										low_count(2 downto 0) & "000000" & iso_track & pm(1 downto 0) & 
										crossing(7 downto 0) & turn(15 downto 0) & "0000000000000000" &
										cluster_a_d(8 downto 0) & cluster_b_d(8 downto 0) & '1';

			-- frames 1-3 have the same format

				frame13(79 downto 0) <= max_trk_d(10 downto 0)  & match_data_d(15 downto 12) &
										high_trk_d(10 downto 0) & match_data_d(11 downto  8) &
										med_trk_d(10 downto 0)  & match_data_d( 7 downto  4) &
										low_trk_d(10 downto 0)  & match_data_d( 3 downto  0) &
										'0' & cluster_a_d(8 downto 0) & cluster_b_d(8 downto 0) & '0';

			-- frames 4 - 6 are the same format

				frame46(79 downto 0) <= max_trk_d(10 downto 0)  & match_data_d(15 downto 12) &
										high_trk_d(10 downto 0) & match_data_d(11 downto  8) &
										med_trk_d(10 downto 0)  & match_data_d( 7 downto  4) &
										low_trk_d(10 downto 0)  & match_data_d( 3 downto  0) &
										"00000000000000000000";

		-- control frame_mux()

			frame_mux(79 downto 0) <= 	frame0(79 downto 0)  when (frame_count="011") else	-- frame0
										frame13(79 downto 0) when (frame_count="100") else	-- frame1
										frame13(79 downto 0) when (frame_count="101") else	-- frame2
										frame13(79 downto 0) when (frame_count="110") else	-- frame3
										frame46(79 downto 0) when (frame_count="000") else	-- frame4
										frame46(79 downto 0) when (frame_count="001") else	-- frame5
										frame46(79 downto 0) when (frame_count="010") else	-- frame6
										X"00000000000000000000";							-- frame7





	-- sequential stuff...

--		count_tracks: process(clock, reset)  change to sync reset Aug-06-04
		count_tracks: process(clock)
		 begin

			if rising_edge(clock) then
			 if (reset='0') then
				max_count(2 downto 0)    <= "000";
				high_count(2 downto 0)   <= "000";
				med_count(2 downto 0)    <= "000";
				low_count(2 downto 0)    <= "000";
				max_cnt_reg(2 downto 0)  <= "000";
				high_cnt_reg(2 downto 0) <= "000";
				med_cnt_reg(2 downto 0)  <= "000";
				low_cnt_reg(2 downto 0)  <= "000";
				frame_count(2 downto 0)  <= "000";
				out_reg(79 downto 0) 	 <= X"00000000000000000000";
				bor_reg					 <= '0';
			 else
				out_reg(79 downto 0) <= frame_mux(79 downto 0);

				if (start='1') then
					frame_count(2 downto 0) <= "000";
				else
					frame_count(2 downto 0) <= frame_count(2 downto 0) + 1;
				end if;

				if (frame_count(2 downto 0)="011") then
					bor_reg <= '1';
				else
					bor_reg <= '0';
				end if;


				if (frame_count(2 downto 0)="001") then
					max_count(2 downto 0)  <= max_cnt_reg(2 downto 0);
					high_count(2 downto 0) <= high_cnt_reg(2 downto 0);
					med_count(2 downto 0)  <= med_cnt_reg(2 downto 0);
					low_count(2 downto 0)  <= low_cnt_reg(2 downto 0);

					max_cnt_reg(2 downto 0)  <= "000";
					high_cnt_reg(2 downto 0) <= "000";
					med_cnt_reg(2 downto 0)  <= "000";
					low_cnt_reg(2 downto 0)  <= "000";
				else
					if (max_trk(10)='1') then
						max_cnt_reg(2 downto 0) <= max_cnt_reg(2 downto 0) + 1;
					end if;

					if (high_trk(10)='1') then
						high_cnt_reg(2 downto 0) <= high_cnt_reg(2 downto 0) + 1;
					end if;

					if (med_trk(10)='1') then
						med_cnt_reg(2 downto 0) <= med_cnt_reg(2 downto 0) + 1;
					end if;

					if (low_trk(10)='1') then
						low_cnt_reg(2 downto 0) <= low_cnt_reg(2 downto 0) + 1;
					end if;
				end if;				

			 end if;
		  end if;
		end process;

	-- instantiate components...

		max_trk_delay: SRL11x16e 
		 	port map(
				d(10 downto 0) 		=> max_trk(10 downto 0),
				q(10 downto 0) 		=> max_trk_d(10 downto 0),
				clock				=> clock,
				ce					=> '1',
				depth(3 downto 0)	=> tdelay(3 downto 0)
			);

		high_trk_delay: SRL11x16e 
		 	port map(
				d(10 downto 0) 		=> high_trk(10 downto 0),
				q(10 downto 0) 		=> high_trk_d(10 downto 0),
				clock				=> clock,
				ce					=> '1',
				depth(3 downto 0)	=> tdelay(3 downto 0)
			);

		med_trk_delay: SRL11x16e 
		 	port map(
				d(10 downto 0) 		=> med_trk(10 downto 0),
				q(10 downto 0) 		=> med_trk_d(10 downto 0),
				clock				=> clock,
				ce					=> '1',
				depth(3 downto 0)	=> tdelay(3 downto 0)
			);

		low_trk_delay: SRL11x16e 
		 	port map(
				d(10 downto 0) 		=> low_trk(10 downto 0),
				q(10 downto 0) 		=> low_trk_d(10 downto 0),
				clock				=> clock,
				ce					=> '1',
				depth(3 downto 0)	=> tdelay(3 downto 0)
			);

		cluster_a_delay: SRL9x16e 
		 	port map(
				d(8 downto 0) 		=> new_cluster_a(8 downto 0),	-- VCccccWww
				q(8 downto 0) 		=> cluster_a_d(8 downto 0),
				clock				=> clock,
				ce					=> '1',
				depth(3 downto 0)	=> cdelay(3 downto 0)
			);

		cluster_b_delay: SRL9x16e 
		 	port map(
				d(8 downto 0) 		=> new_cluster_b(8 downto 0),	-- VCccccWww
				q(8 downto 0) 		=> cluster_b_d(8 downto 0),
				clock				=> clock,
				ce					=> '1',
				depth(3 downto 0)	=> cdelay(3 downto 0)
			);

		match_delay: SRL16x16e 
		 	port map(
				d(15 downto 0) 		=> match_data(15 downto 0),
				q(15 downto 0) 		=> match_data_d(15 downto 0),
				clock				=> clock,
				ce					=> '1',
				depth(3 downto 0)	=> mdelay(3 downto 0)
			);


end L2_packer_arch;
