-- L2CFT_format.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- 	format the L2 CFT record.  build up the header frames
-- 	and data frames.  CFT number of objects = number of found tracks.  valid range is 0-24.

-- 	the relatively static stuff like pm, oct, sect, turn, etc. should be stable
-- 	before frame_sel is asserted.  these signals should be stable
-- 	the entire time that the CFT record is being generated.

-- 	the timing of the frame_sel inputs MUST be synchronized to when the
--	data comes out of the L2_trksort and L2_clustram modules.  CFT tracks must be presented
--  to this module in order of decreasing Pt and with no gaps.

--		frame_sel		output
--		---------		-----------------------
--		01				CFT L2 header frame
--		10				Turn and Crossing frame
--		11				CFT L2 data frame
--		others			zeros

-- *** revision history ***
--
--  8 Apr 2002: created, functionally simulated.
-- 24 Sep 2002: stripped down, CFT functionality only.
-- 26 Sep 2002: if the cluster match is not set, zero out the centroid field
-- 16 Oct 2002:	add input registers.  note: tvalid and cvalid inputs are not used here!
-- 17 Oct 2002: fix bug with R bit.  bring tvalid and cvalid back in.
-- 22 Oct 2002: fixed bug with centroid register.
-- 23 Oct 2002: support fake L2CFT record generation.  fake record is a header with OCT=0, PM=10,
--              number of objects=0, turn=0x1234, crossing=0x56
-- 25 Apr 2003:	send fake L2CFT record with real tick/turn and fake tracks.
-- 13 May 2003:  consistant curve 1 = CCW

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity L2CFT_format is
	port(
		reset:		in	std_logic;		-- active low, async.
		clock:		in	std_logic;		-- 53MHz
		send_fake:	in	std_logic;		-- produce a fake L2CFT record

		frame_sel:	in	std_logic_vector( 1 downto 0);	-- select the type of word to be output

		-- "static" L2 record info

		pm:			in	std_logic_vector( 1 downto 0);	-- pass and mark bits
		octant:		in	std_logic_vector( 2 downto 0);	-- octant number (0-7)
		sector:		in	std_logic_vector( 3 downto 0);	-- relative sector within an octant (0-9)
		turn:		in	std_logic_vector(15 downto 0);	-- turn counter
		cross:		in	std_logic_vector( 7 downto 0);	-- crossing counter
		iso:		in	std_logic;						-- a single isolated track
		track_cnt:	in	std_logic_vector( 4 downto 0);	-- number of found tracks to be reported for L2CFT [0-24]

		-- raw tracks and cluster information from L2_TRKSORT and L2_CLUSTRAM modules

		cvalid:		in	std_logic;						-- cluster data from the L2engine
		centroid:	in	std_logic_vector(4 downto 0);
		tvalid:		in	std_logic;						-- track data from the L2engine
		curve:		in	std_logic;
		pt:			in	std_logic_vector(1 downto 0);
		extpt:		in 	std_logic_vector(2 downto 0);
		phi:		in 	std_logic_vector(5 downto 0);
		cm:			in	std_logic;

		dout:		out	std_logic_vector(27 downto 0)	-- record output, parity has NOT been addeded, yet.
	);
end L2CFT_format;

architecture L2CFT_format_arch of L2CFT_format is

	-- declare registers...

		signal dout_reg:	std_logic_vector(27 downto 0);

		signal frame_sel_reg:	std_logic_vector(1 downto 0);
		signal centroid_reg:	std_logic_vector(4 downto 0);
		signal curve_reg:		std_logic;
		signal pt_reg:			std_logic_vector(1 downto 0);
		signal phi_reg:			std_logic_vector(5 downto 0);
		signal extpt_reg:		std_logic_vector(2 downto 0);
		signal cm_reg:			std_logic;
		signal tvalid_reg:		std_logic;
		signal cvalid_reg:		std_logic;
		signal fake_count_reg:	std_logic_vector(2 downto 0);

	-- declare signals...

		signal frame_mux:	std_logic_vector(27 downto 0);
		signal turn_frame:	std_logic_vector(27 downto 0);

		-- signal cps_ob_hp:	std_logic_vector( 1 downto 0);
		-- signal cps_ob_vp:	std_logic_vector( 2 downto 0);

		signal cft_ob_hp:	std_logic_vector( 1 downto 0);
		signal cft_ob_vp:	std_logic_vector( 2 downto 0);

		signal cent:		std_logic_vector( 4 downto 0);

		signal cft_header_frame:  std_logic_vector(27 downto 0);

		signal cft_data_frame:  std_logic_vector(27 downto 0);

		signal r_bit: 	std_logic;
		
		signal fake_header_frame:  	std_logic_vector(27 downto 0);
		-- fake turn frame is same as real turn frame
		signal fake_trk1:	std_logic_vector(27 downto 0);
		signal fake_trk2:	std_logic_vector(27 downto 0);
		signal fake_trk3:	std_logic_vector(27 downto 0);
		signal fake_trk4:	std_logic_vector(27 downto 0);

begin

	-- combinatorial stuff...

			dout(27 downto 0)	<= dout_reg(27 downto 0);

		-- calculate the vertical and hortizonal EVEN parity fields for the
		-- CFT number of tracks.

			cft_ob_hp(1)	<= 	'1' when (track_cnt(4 downto 3)="01") else
								'1' when (track_cnt(4 downto 3)="10") else
								'0';

			cft_ob_hp(0)	<= 	'1' when (track_cnt(2 downto 0)="001") else
								'1' when (track_cnt(2 downto 0)="010") else
								'1' when (track_cnt(2 downto 0)="100") else
								'1' when (track_cnt(2 downto 0)="010") else
								'1' when (track_cnt(2 downto 0)="111") else
								'0';

			cft_ob_vp(2)	<=	track_cnt(2);

			cft_ob_vp(1)	<=	track_cnt(4) xor track_cnt(1);

			cft_ob_vp(0)	<=	track_cnt(3) xor track_cnt(0);

		-- L2CFT header frame (horizontal parity bit will be modified later)

			cft_header_frame(27 downto 0)	<= 	"1110" & pm(1 downto 0) & octant(2 downto 0) & cft_ob_vp(2 downto 0) &
												'0' & track_cnt(4 downto 3) & track_cnt(2 downto 0) & cft_ob_hp(1 downto 0) &
												"00101" & "000";

		-- build up the fake L2CFT record...

			fake_header_frame(27 downto 0)	<= 	"111010" & octant(2 downto 0) & "1000001000100101000";
			-- fake turn frame is same as real turn frame...
			fake_trk1(27 downto 0) <= "00000011" & sector(3 downto 0) & "1010000000001110";
			fake_trk2(27 downto 0) <= "00000000" & sector(3 downto 0) & "0000100100011010";
			fake_trk3(27 downto 0) <= "00001101" & sector(3 downto 0) & "1011001101010000";
			fake_trk4(27 downto 0) <= "00000000" & sector(3 downto 0) & "0011101000101100";
			-- parity frame will be added automatically...


		-- the R bit is set if a CFT track matches to a cluster who's centroid is OUTSIDE of the home sector.
		-- just take the XNOR of the two MSbs in the cluster centoid field.  see engineering note 2002-01-22a
		-- for more information.

--			r_bit <= (centroid_reg(4) xnor centroid_reg(3)) and tvalid_reg and cvalid_reg;
--	 3/10/2005 cvalid_reg does	not equal to there is a match S. X. Wu
			r_bit <= (centroid_reg(4) xnor centroid_reg(3)) and tvalid_reg and cm_reg;


		-- if the cluster_match bit is not set, blank out the reported centroid field.  The reported
		-- centroid should be in the range of 0-15.
			cent(4 downto 0) <= (centroid_reg(4 downto 0)-"01000") when (cm_reg='1') else 
								"00000";

    	-- build the L2 CFT data frame from static variables (sector, etc.) and fields coming from the
		-- L2engine.  (horizontal parity bit will be modified later)

			cft_data_frame(27 downto 0)	<= 	"0000" & cent(3 downto 0) & sector(3 downto 0) &
											cm_reg & '0' & curve_reg & pt_reg(1 downto 0) & extpt_reg(2 downto 0) &
											r_bit & phi_reg(5 downto 0) & iso;

		-- turn and crossing frame (horizontal parity bit will be modified later)

			turn_frame(27 downto 0)	<= "0000" & turn(15 downto 0) & cross(7 downto 0);

		-- large mux to select which frame to output

			frame_mux(27 downto 0)	<=	cft_header_frame(27 downto 0) 	when ((frame_sel_reg(1 downto 0)="01") and send_fake='0') else
										turn_frame(27 downto 0) 		when ((frame_sel_reg(1 downto 0)="10") and send_fake='0') else
										cft_data_frame(27 downto 0) 	when ((frame_sel_reg(1 downto 0)="11") and send_fake='0') else

										fake_header_frame(27 downto 0)	when (fake_count_reg(2 downto 0)="000" and send_fake='1') else
										turn_frame(27 downto 0) 	    when (fake_count_reg(2 downto 0)="001" and send_fake='1') else
										fake_trk1(27 downto 0) 			when (fake_count_reg(2 downto 0)="010" and send_fake='1') else
										fake_trk2(27 downto 0) 			when (fake_count_reg(2 downto 0)="011" and send_fake='1') else
										fake_trk3(27 downto 0) 			when (fake_count_reg(2 downto 0)="100" and send_fake='1') else
										fake_trk4(27 downto 0) 			when (fake_count_reg(2 downto 0)="101" and send_fake='1') else
										X"0000000";

	-- sequential stuff...

--		register_stuff: process(clock, reset)	 change to sync reset Aug-06-04
		register_stuff: process(clock)
			begin
			  if rising_edge(clock) then
				 if (reset='0') then
					dout_reg(27 downto 0)		<= X"0000000";
					frame_sel_reg(1 downto 0) 	<= "00";
					centroid_reg(4 downto 0)	<= "00000";
					curve_reg					<= '0';
					pt_reg(1 downto 0)			<= "00";
					extpt_reg(2 downto 0)		<= "000";
					phi_reg(5 downto 0)			<= "000000";
					cm_reg						<= '0';
					tvalid_reg					<= '0';
					cvalid_reg					<= '0';
					fake_count_reg(2 downto 0)	<= "111";
				 else
					dout_reg(27 downto 0)		<= frame_mux(27 downto 0);
					frame_sel_reg(1 downto 0) 	<= frame_sel(1 downto 0);
					centroid_reg(4 downto 0)	<= centroid(4 downto 0);
					curve_reg					<= curve;
					pt_reg(1 downto 0)			<= pt;
					phi_reg(5 downto 0)			<= phi(5 downto 0);
					extpt_reg(2 downto 0)		<= extpt;
					cm_reg						<= cm;
					tvalid_reg					<= tvalid;
					cvalid_reg					<= cvalid;

					if (send_fake='1' and frame_sel(1 downto 0)="01") then
						fake_count_reg(2 downto 0)	<= "000";
					elsif (fake_count_reg(2 downto 0) /= "111") then
						fake_count_reg(2 downto 0)	<= fake_count_reg(2 downto 0) + 1;
					end if;

				 end if;
			  end if;
		end process;


	-- instantiate components...

end L2CFT_format_arch;
