-- tick_turn.vhd
-- jamieson olsen

-- this module looks at four control bits (FX, CR, BOR, and L1) and determines
-- the correct tick and turn numbers.  It also provides a status bit called
-- locked that lets the user know that the tick and turn numbers are valid.

-- BOR: beginning of record, active high.  marks the beginning of L1 and L2 records.
-- L1R:	L1 record.  if the incoming record is NOT a L1 record, the FX and CR bits are
--		not valid and should be ignored.  This signal is sampled when BOR is asserted.
-- FX: 	first crossing, active high.  sampled only when BOR is asserted.
-- CR: 	CFT reset, active high.  aka SCL init.  sampled only when BOR is asserted.

-- BOR	________________/-------\_______________________	(asserted for 1 clock)
-- L1R	================x valid X=======================
-- FX	================x valid X=======================
-- CR	================x valid X=======================

-- To achive lock, this module *must* see a CR pulse followed by an FX pulse.  After 
-- it achieves lock the tick and turn counts are valid.  If FX is not present (i.e. if a L2
-- record displaced the L1 record normally carrying the FX bit) this module will ride it 
-- out, not dropping lock, and maintaining the tick and turn counts.

-- if this module is not locked, then the tick and turn counts are ZERO.

-- *** revision history ***
--	15 Nov 2002		created
--	 5 Dec 2002		add L1 input.  now counts CLOCK cycles, not BOR.  
--					now the module can "ride out" L2 records and missing FX bits.
--  31 Jan 2003		fix bug where module stays locked even though FX is arriving at the wrong times.
--					now this module will drop the locked bit and zero out tick and turn
--					if FX comes at the wrong time.  add locked_reg to make probing easier.
--   5 Feb 2003		loosen up the criteria for fx_wrongtime, register that signal to make probing easier.
--   5 Feb 2003		found major bug in tick/turn: wrongtime was looking at the TURN count, not the TICK
--					count.  Really dumb...
--  18 Feb 2003		once locked, if FX arrives at the wrong time pulse the "bad_fx" output but
--					keep the tick and turn counters going.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity tick_turn is
    port (
		bor:	in	std_logic;	-- marks the first frame of a record (could be L1 or L2)
		l1:		in	std_logic;	-- set if the BOR is for a L1 record
		fx:		in 	std_logic;	-- First Crossing
		cr:		in	std_logic;	-- CFT_RESET aka SCL_INIT 

		clock:	in 	std_logic;	-- 53MHz
		reset:	in	std_logic;	-- active low

		tick:	out	std_logic_vector( 7 downto 0);
		turn:	out	std_logic_vector(15 downto 0);
		bad_fx:	out	std_logic;		-- pulsed if FX comes at the wrong time.
		lock:	out	std_logic		-- set if 
    );
end tick_turn;

architecture tick_turn_arch of tick_turn is

	-- define registers...

		signal frame_reg:	std_logic_vector( 2 downto 0);	-- counts the frames
		signal turn_reg:	std_logic_vector(15 downto 0);
		signal tick_reg:	std_logic_vector( 7 downto 0);
		signal fx_reg:		std_logic;
		signal cr_reg:		std_logic;
		signal bor_reg:		std_logic;
		signal l1_reg:		std_logic;
		signal locked_reg:	std_logic;
		signal wrong_reg:	std_logic;

	-- define signals....

		type state_type is (rst, wait4cr, wait4fx, new_turn, locked);
		signal state: state_type;

		signal fx_wrong_time:	std_logic;

 begin

	-- combinatorial stuff...

		-- assign the outputs...

			turn(15 downto 0)   <= 	turn_reg(15 downto 0) when (state=locked) else
									X"0000";

			tick(7 downto 0)    <= 	tick_reg( 7 downto 0) when (state=locked) else
									X"00";


			lock				<= locked_reg;
			bad_fx				<= wrong_reg;
	
		-- if FX arrives at the wrong time, assert this signal.  it will become the bad_fx output.

			fx_wrong_time <= '1' when (	bor_reg='1' and 
										fx_reg='1' and 
                                       	(frame_reg(2 downto 0)="101") and 
										(tick_reg(7 downto 0) /= X"06")) else '0';


	-- sequential stuff...

--	reg_stuff: process(clock, reset)	 change to sync reset Aug-06-04
	reg_stuff: process(clock)
	 begin

	  if rising_edge(clock) then
		 if (reset='0') then
			bor_reg <= '0';
			fx_reg  <= '0';
			cr_reg  <= '0';
			l1_reg	<= '0';
			tick_reg(7 downto 0) <= X"00";
			turn_reg(15 downto 0) <= X"0000";
			frame_reg(2 downto 0) <= "000";
			locked_reg <= '0';
			wrong_reg  <= '0';
		 else
			bor_reg <= bor;
			fx_reg  <= fx;
			cr_reg  <= cr;
			l1_reg 	<= l1;

			if (state=locked) then
				locked_reg <= '1';
			else
				locked_reg <= '0';
			end if;

			-- if locked and FX comes at the wrong time, pulse the wrong_reg

			if (state=locked) then
				wrong_reg <= fx_wrong_time;
			else
				wrong_reg <= '0';
			end if;

			-- frame_reg is a modulo 7 counter...			

				if (state=new_turn) then
					frame_reg(2 downto 0)	<= "000";
				elsif (frame_reg(2 downto 0)="110") then
					frame_reg(2 downto 0) <= "000";
				else
					frame_reg(2 downto 0) <= frame_reg(2 downto 0) + 1;
				end if;

			-- tick_reg counts from 0 to 158.  It is reset to 7 in state new_turn and 
			-- it increments whenever frame_reg=6.

				if (state=new_turn) then
					tick_reg(7 downto 0) <= X"07";
				elsif (tick_reg(7 downto 0)=X"9E" and frame_reg(2 downto 0)="110") then
					tick_reg(7 downto 0) <= X"00";
				elsif (frame_reg(2 downto 0)="110") then
					tick_reg(7 downto 0) <= tick_reg(7 downto 0) + 1;
				end if;

			-- turn_reg is initialized to 1 in state new_turn.  it increments every time
			-- tick_reg=158 and frame_reg=6.

				if (state=new_turn) then
					turn_reg(15 downto 0) <= X"0001";
				elsif (tick_reg(7 downto 0)=X"9E" and frame_reg(2 downto 0)="110") then
					turn_reg(15 downto 0) <= turn_reg(15 downto 0) + 1;
				end if;

	
		 end if;	 
	  end if;
 	end process;



--	fsm: process(clock, reset)	 	 change to sync reset Aug-06-04
	fsm: process(clock)
	 begin
		if rising_edge(clock) then
		 if (reset='0') then
			state <= rst;
		 else
			case state is
				when rst =>
 					state <= wait4cr;
				when wait4cr =>
					if (l1_reg='1' and bor_reg='1' and cr_reg='1') then
						state <= wait4fx;
					else
						state <= wait4cr;
					end if;
				when wait4fx =>
					if (cr_reg='0' and fx_reg='1' and bor_reg='1' and l1_reg='1') then
						state <= new_turn;
					else
						state <= wait4fx;
					end if;
				when new_turn =>
					state <= locked;
				when locked =>
					if (l1_reg='1' and bor_reg='1' and cr_reg='1') then
						state <= wait4fx;
					--elsif (wrong_reg='1') then
					--	state <= wait4cr;
					else
						state <= locked;
					end if;		
				when others =>
					state <= rst;
			end case;
		 end if;
	  end if;
	end process;


end tick_turn_arch;
