-- parity.vhd
-- jamieson olsen <jamieson@fnal.gov>

-- 		calculate and insert both horizontal and vertical EVEN parity bits into the
-- 		outgoing L1/L2 stream to CTOC and other flavours of double-wide boards.

--		horizontal parity is calculated for a single frame, bits [23..0].  this
-- 		parity bit is EVEN parity (XNOR based) and seeks to make the number
--		of 1's EVEN.  Likewise for the vertical parity word.  an example...
--
--		222 2 22221111111111                 222 2 22221111111111
--		765 4 321098765432109876543210	     765 4 321098765432109876543210
--		--- - ------------------------       --- - ------------------------
--		111 h ........................       111 1 ..........1.............
--		000 h ........................       000 1 .......1.....1....1.....
--		000 h ........................       000 1 .............1..........
--		000 h ........................       000 . ........................
--		000 h ........................       000 . .........1...1..........
--		000 h ........................       000 1 .......1................
--		000 0 vvvvvvvvvvvvvvvvvvvvvvvv       000 0 .........11..1....1.....

--		L1 records have the three LSb's of the header = "111".  L1 records are
--		*always* 7 frames long.  L2 records have "000" as the three LSb's of
--		the header frame.  L2 records are variable length, so look at the
--		"number of objects" field (bits 15..10 of the header) to calculate the
--		total number of frames in the record.  For L2 records the calculation is
--		total frames = #objects + 1 header frame + 1 turn/crossing frame
--		+ 1 trailer frame

--		NOTE: this module needs to see a dummy parity word on the input bus for 
--		all record types.  The contents of the dummy frame are NOT used in the 
--		vertical parity calculation.

--		1: 0xE000007	<= L1 header
--		2: 0x0000000
--		3: 0x0000000
--		4: 0x0000000
--		5: 0x0000000
--		6: 0x0000000
--		7: 0x0000000	<= dummy parity frame (contents ignored)
--		1: 0xE000007	<= L1 header
--		2: 0x0000000
--		3: 0x0000000
--		4: 0x0000000
--		5: 0x0000000
--		6: 0x0000000
--		7: 0x0000000	<= dummy parity frame (contents ignored)



-- ** revision history **

-- 05 Jun 2001:	  	created JTO tested OK for occupancy test

-- 09 Apr 2002:		add smarts to look at the header frame and determine
--					how long the record is.	see eng. note 2002-04-09b for details.

-- 26 Jun 2002: 	add feature to send test pattern (johnson counter) over the 
--					28-bit CTOC output instead of data.

-- 07 Oct 2002:		add ctoc_zero input.  sends nothing but zeros when asserted.

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity parity is
	port(
		reset:		in 	std_logic;						-- active low async
		clock:		in 	std_logic;						-- dfe board clock 53mhz
		ctoc_patt:	in	std_logic;						-- generate test pattern, active high
		ctoc_zero:	in	std_logic;						-- send zeros when set
		d:			in	std_logic_vector(27 downto 0);
		q:			out	std_logic_vector(27 downto 0)
	);
end parity;

architecture parity_arch of parity is

	-- define signals...

	signal even_nib:	std_logic_vector(5 downto 0);
	signal even_byte:	std_logic_vector(2 downto 0);
	signal hpar_bit:	std_logic;
	--	signal l2rec:		std_logic;						-- L2 record coming in, removed 7 May 04

	-- define registers...

	signal out_reg:		std_logic_vector(27 downto 0);
	signal in_reg:		std_logic_vector(27 downto 0);
	signal vpar1_reg:	std_logic_vector(27 downto 0);
	signal vpar2_reg:	std_logic_vector(27 downto 0);
	signal hpar_reg:	std_logic_vector(27 downto 0);
	signal count_reg:	std_logic_vector( 5 downto 0);

	signal johnson_reg:	std_logic_vector(27 downto 0);

 begin

	-- combinatorial stuff...

-- out_reg moved to top level back_chip.vhd Aug-31-04
--	q(27 downto 0)	<= out_reg(27 downto 0);
q <= johnson_reg when ctoc_patt = '1' else -- send the johnson counter pattern
     x"0000000" when ctoc_zero = '1' else	-- send nulls
	  vpar2_reg when count_reg = "000001" else -- send vertical parity frame
	  x"0000000" when count_reg = "000000" else -- pad out record with zeros
	  hpar_reg;	-- send header or data frame


	even_nib(0) <= 	'1' when (in_reg( 3 downto 0)="0000") else	-- set if the least significant nibble
				  	'1' when (in_reg( 3 downto 0)="0011") else	-- has an even number of 1's in it.
				  	'1' when (in_reg( 3 downto 0)="0101") else
				  	'1' when (in_reg( 3 downto 0)="0110") else
				  	'1' when (in_reg( 3 downto 0)="1001") else
				  	'1' when (in_reg( 3 downto 0)="1010") else
				  	'1' when (in_reg( 3 downto 0)="1100") else
				  	'1' when (in_reg( 3 downto 0)="1111") else
					'0';

	even_nib(1) <= 	'1' when (in_reg( 7 downto 4)="0000") else
				  	'1' when (in_reg( 7 downto 4)="0011") else
				  	'1' when (in_reg( 7 downto 4)="0101") else
				  	'1' when (in_reg( 7 downto 4)="0110") else
				  	'1' when (in_reg( 7 downto 4)="1001") else
				  	'1' when (in_reg( 7 downto 4)="1010") else
				  	'1' when (in_reg( 7 downto 4)="1100") else
				  	'1' when (in_reg( 7 downto 4)="1111") else
					'0';

	even_nib(2) <= 	'1' when (in_reg(11 downto 8)="0000") else
				  	'1' when (in_reg(11 downto 8)="0011") else
				  	'1' when (in_reg(11 downto 8)="0101") else
				  	'1' when (in_reg(11 downto 8)="0110") else
				  	'1' when (in_reg(11 downto 8)="1001") else
				  	'1' when (in_reg(11 downto 8)="1010") else
				  	'1' when (in_reg(11 downto 8)="1100") else
				  	'1' when (in_reg(11 downto 8)="1111") else
					'0';

	even_nib(3) <= 	'1' when (in_reg(15 downto 12)="0000") else
				  	'1' when (in_reg(15 downto 12)="0011") else
				  	'1' when (in_reg(15 downto 12)="0101") else
				  	'1' when (in_reg(15 downto 12)="0110") else
				  	'1' when (in_reg(15 downto 12)="1001") else
				  	'1' when (in_reg(15 downto 12)="1010") else
				  	'1' when (in_reg(15 downto 12)="1100") else
				  	'1' when (in_reg(15 downto 12)="1111") else
					'0';

	even_nib(4) <= 	'1' when (in_reg(19 downto 16)="0000") else
				  	'1' when (in_reg(19 downto 16)="0011") else
				  	'1' when (in_reg(19 downto 16)="0101") else
				  	'1' when (in_reg(19 downto 16)="0110") else
				  	'1' when (in_reg(19 downto 16)="1001") else
				  	'1' when (in_reg(19 downto 16)="1010") else
				  	'1' when (in_reg(19 downto 16)="1100") else
				  	'1' when (in_reg(19 downto 16)="1111") else
					'0';

	even_nib(5) <= 	'1' when (in_reg(23 downto 20)="0000") else
				  	'1' when (in_reg(23 downto 20)="0011") else
				  	'1' when (in_reg(23 downto 20)="0101") else
				  	'1' when (in_reg(23 downto 20)="0110") else
				  	'1' when (in_reg(23 downto 20)="1001") else
				  	'1' when (in_reg(23 downto 20)="1010") else
				  	'1' when (in_reg(23 downto 20)="1100") else
				  	'1' when (in_reg(23 downto 20)="1111") else
					'0';


	even_byte(0)	<=	'1' when (even_nib(1 downto 0)="00") else		-- set this bit if the least signif. byte
						'1' when (even_nib(1 downto 0)="11") else		-- has an even number of 1's
						'0';

	even_byte(1)	<=	'1' when (even_nib(3 downto 2)="00") else		-- set this bit if the middle byte
						'1' when (even_nib(3 downto 2)="11") else		-- has an even number of 1's
						'0';

	even_byte(2)	<=	'1' when (even_nib(5 downto 4)="00") else		-- set this bit if the most signif. byte
						'1' when (even_nib(5 downto 4)="11") else		-- has an even number of 1's
						'0';

	hpar_bit	<=	'1' when (even_byte(2 downto 0)="000") else     	-- set the hpar_bit if the number of
					'1' when (even_byte(2 downto 0)="011") else			-- 1's in the three bytes is ODD.
					'1' when (even_byte(2 downto 0)="101") else
					'1' when (even_byte(2 downto 0)="110") else
					'0';

	-- sequential stuff...

--	maintain_reg: process(clock, reset) change to sync reset Aug-06-04
	maintain_reg: process(clock)
 	 begin

		if rising_edge(clock) then
		 if (reset='0') then
			in_reg(27 downto 0)			<= X"0000000";
-- out_reg moved to top level back_chip.vhd Aug-31-04
--			out_reg(27 downto 0)		<= X"0000000";
			hpar_reg(27 downto 0)		<= X"0000000";
			vpar1_reg(27 downto 0)		<= X"0000000";
			vpar2_reg(27 downto 0)		<= X"0000000";
			count_reg(5 downto 0)		<= "000000";
			johnson_reg(27 downto 0)	<= X"0000000";
		 else
			-- register the input

				in_reg(27 downto 0)		<= d(27 downto 0);

			-- register to horizontal parity word

				hpar_reg(27 downto 0)	<= in_reg(27 downto 25) & hpar_bit & in_reg(23 downto 0);

			-- 	calculate the vertical parity word.  if in_reg contains a header frame, then
			--	load vpar_reg with in_reg.  on every other frame XOR in_reg with vpar_reg.

    			if (in_reg(27 downto 25)="111") then
    				vpar1_reg(27 downto 0)	<= "0000" & in_reg(23 downto 0);
    			else
    				vpar1_reg(27 downto 0)	<= "0000" & (in_reg(23 downto 0) xor vpar1_reg(23 downto 0));
    			end if;

			-- delay the calculated vertical parity frame by one clock cycle so that the 
			-- dummy parity frame fed into the input is NOT included in the vertical parity 
			-- calculation.

				vpar2_reg(27 downto 0)	<= vpar1_reg(27 downto 0);

			-- count_reg is loaded on the clock cycle after the header is
			-- loaded into in_reg.  count_reg will count down to zero and stay
			-- there until it's reloaded by a new header.  send the trailer
			-- vertical parity word when count_reg = 1.

    			if (in_reg(27 downto 25)="111" and in_reg(2 downto 0)="000") then		-- L2 header
    				count_reg(5 downto 0)	<= in_reg(15 downto 10) + 3;				-- load the counter with the total number of L2 frames

    			elsif (in_reg(27 downto 25)="111" and in_reg(2 downto 0)="111") then	-- L1 header
    				count_reg(5 downto 0)	<= "000111";								-- load the counter with the total number of L1 frames

    			elsif (count_reg(5 downto 0) /= "000000") then							-- count down to zero and hold there.
    				count_reg <= count_reg - 1;
    			end if;


			-- make the johnson counter

				johnson_reg(27 downto 1) 	<= johnson_reg(26 downto 0);
				johnson_reg(0)				<= not johnson_reg(27);

			-- register the output, based on the contents of count_reg and test input.
			-- out_reg should be packed into the IOBs.  ctoc pattern has priority over
			-- ctoc_zero.

-- out_reg moved to top level back_chip.vhd Aug-31-04
--				if (ctoc_patt='1') then			-- send the johnson counter pattern
--					out_reg(27 downto 0) <= johnson_reg(27 downto 0);

--				elsif (ctoc_zero='1') then		-- send nulls
--					out_reg(27 downto 0) <= X"0000000";

--				elsif (count_reg(5 downto 0)="000001") then		-- send vertical parity frame
--					out_reg(27 downto 0) <= vpar2_reg(27 downto 0);

--				elsif (count_reg(5 downto 0)="000000") then		-- pad out record with zeros
--					out_reg(27 downto 0) <= X"0000000";
--				else
--					out_reg(27 downto 0) <= hpar_reg(27 downto 0);	-- send header or data frame
--				end if;

		 end if;
	  end if;
	end process;

end parity_arch;
