-- cps_edge.vhd
-- determine the "inner" and "outer" edges of the CPS cluster string.  
-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***

-- 18 march 2002: created, functionally simulated.

-- two cluster finders (A and B) operate in parallel, each working on a 16-bit 
-- long string.  finder A starts in the center of the home sector and finds clusters
-- in the INCREASING PHI direction.  finder b starts in the center of the home sector
-- and finds clusters in the DECREASING PHI direction.

-- (see engineering note 2002-01-22a for details)

-- CPS(7..0) are from PREVIOUS
-- CPS(23..8) are from HOME
-- CPS(31..24) are from NEXT 

-- CPS index numbers *always* increase with phi when viewed looking 
-- NORTH at the SOUTH face of the fiber tracker.

-- <<< finder A <<< >>> finder B >>>
-- 3322222222221111 1111110000000000
-- 1098765432109876 5432109876543210
-- ....111...11.1.. ...11....1...... raw CPS hits
-- ......1....1.1.. ................ a inner edge
-- ....1.....1..1.. ................ a outer edge
-- ................ ...1.....1...... b inner edge
-- ................ ....1....1...... b outer edge

-- ...1111......111 111...11...11111 raw CPS hits
-- ......1........1 ................ a inner edge
-- ...1.........1.. ................ a outer edge
-- ................ 1.....1....1.... b inner edge
-- ................ ..1....1.......1 b outer edge
-- 1111110000000000 1111110000000000 a index, b index
-- 5432109876543210 5432109876543210

library ieee;
use ieee.std_logic_1164.all;

entity cps_edge is
 port (
	    cps:    in  std_logic_vector(31 downto 0);
	    a_in:  	out std_logic_vector(15 downto 0);
		a_out:	out std_logic_vector(15 downto 0);
	    b_in:  	out std_logic_vector(15 downto 0);
		b_out:	out std_logic_vector(15 downto 0)
    );
end cps_edge;

architecture cps_edge_arch of cps_edge is

 begin

    -- combinatorial stuff...

		-- look for pattern "10"

		a_in(0)		<=	cps(16);
		a_in(1)		<= 	cps(17) and (not cps(16));
		a_in(2)		<= 	cps(18) and (not cps(17));
		a_in(3)		<= 	cps(19) and (not cps(18));
		a_in(4)		<= 	cps(20) and (not cps(19));
		a_in(5)		<= 	cps(21) and (not cps(20));
		a_in(6)		<= 	cps(22) and (not cps(21));
		a_in(7)	    <= 	cps(23) and (not cps(22));
		a_in(8)		<= 	cps(24) and (not cps(23));
		a_in(9)		<= 	cps(25) and (not cps(24));
		a_in(10)	<= 	cps(26) and (not cps(25));
		a_in(11)	<= 	cps(27) and (not cps(26));
		a_in(12)	<= 	cps(28) and (not cps(27));
		a_in(13)	<= 	cps(29) and (not cps(28));
		a_in(14)	<= 	cps(30) and (not cps(29));
		a_in(15)	<= 	cps(31) and (not cps(30));

		-- look for pattern "01"

		a_out(0)	<=	cps(16) and (not cps(17));
		a_out(1)	<= 	cps(17) and (not cps(18));
		a_out(2)	<= 	cps(18) and (not cps(19));
		a_out(3)	<= 	cps(19) and (not cps(20));
		a_out(4)	<= 	cps(20) and (not cps(21));
		a_out(5)	<= 	cps(21) and (not cps(22));
		a_out(6)	<= 	cps(22) and (not cps(23));
		a_out(7)	<= 	cps(23) and (not cps(24));
		a_out(8)	<= 	cps(24) and (not cps(25));
		a_out(9)	<= 	cps(25) and (not cps(26));
		a_out(10)	<= 	cps(26) and (not cps(27));
		a_out(11)	<= 	cps(27) and (not cps(28));
		a_out(12)	<= 	cps(28) and (not cps(29));
		a_out(13)	<= 	cps(29) and (not cps(30));
		a_out(14)	<= 	cps(30) and (not cps(31));
		a_out(15)	<= 	cps(31);

		-- look for pattern "01"

		b_in(0)		<=	cps(0)  and (not cps(1));
		b_in(1)		<= 	cps(1)  and (not cps(2));
		b_in(2)		<= 	cps(2)  and (not cps(3));
		b_in(3)		<= 	cps(3)  and (not cps(4));
		b_in(4)		<= 	cps(4)  and (not cps(5));
		b_in(5)		<= 	cps(5)  and (not cps(6));
		b_in(6)		<= 	cps(6)  and (not cps(7));
		b_in(7)	    <= 	cps(7)  and (not cps(8));
		b_in(8)		<= 	cps(8)  and (not cps(9));
		b_in(9)		<= 	cps(9)  and (not cps(10));
		b_in(10)	<= 	cps(10) and (not cps(11));
		b_in(11)	<= 	cps(11) and (not cps(12));
		b_in(12)	<= 	cps(12) and (not cps(13));
		b_in(13)	<= 	cps(13) and (not cps(14));
		b_in(14)	<= 	cps(14) and (not cps(15));
		b_in(15)	<= 	cps(15);

		-- look for pattern "10"

		b_out(0)	<=	cps(0);
		b_out(1)	<= 	cps(1)  and (not cps(0));
		b_out(2)	<= 	cps(2)  and (not cps(1));
		b_out(3)	<= 	cps(3)  and (not cps(2));
		b_out(4)	<= 	cps(4)  and (not cps(3));
		b_out(5)	<= 	cps(5)  and (not cps(4));
		b_out(6)	<= 	cps(6)  and (not cps(5));
		b_out(7)	<= 	cps(7)  and (not cps(6));
		b_out(8)	<= 	cps(8)  and (not cps(7));
		b_out(9)	<= 	cps(9)  and (not cps(8));
		b_out(10)	<= 	cps(10) and (not cps(9));
		b_out(11)	<= 	cps(11) and (not cps(10));
		b_out(12)	<= 	cps(12) and (not cps(11));
		b_out(13)	<= 	cps(13) and (not cps(12));
		b_out(14)	<= 	cps(14) and (not cps(13));
		b_out(15)	<= 	cps(15) and (not cps(14));

    -- sequential stuff...

end cps_edge_arch;
