-- CFT track sorter pipeline
-- top level entity.

-- jamieson olsen <jamieson@fnal.gov>

-- *** revision history ***

-- 14 Jul 1999:	sync design runs at 70MHz+ uses a stack to store extra 
--              tracks and this results in scrambling the order of the found
--              tracks.

-- 03 Jan 2002: use a pair of dual port rams instead of the stack.  this does
--              not introduce any scrabling effects.  Now the pipeline has 
--              an "elastic" quality.  Can find the two edge tracks per 22-bit
--              BLOCK.  Tracks are reported in order!  This backend logic is 
--              now generic and can be used for MAX, HIGH, and MEDIUM designs.

-- 07 Jan 2002: register the EOF/load signal to allow the synth tool to 
--              duplicate registers.  Load must be asserted one clock cycle
--              *BEFORE* valid inputs on MATRIX_D().

-- 25 feb 2002: change from tri_mux352 to a mux built out of LUTs.

-- 21 May 2002: add start bit diagnostic output.

-- 05 Aug 2002:	early track output bus for L1muon added.  CTOC output bus latency
--				reduced by 1 clock cycle.  CTOC output bus scaled back to make room
--				for separate L1muon track bus going to U1.  See engineering note 
--				2002-08-05a for details.  NOTE: 2 tracks/blocks are not resolved 
--				for L1muon; no changes to CTOC output bus.  Output bus format same
--				for L1muon and CTOC busses: VCEePppppp.  The start bit is set when
--				the first L1muon track is output.

-- revised 24 March 2004: cleanup for XST, removed trk12_reg (unconnected)


-- how it works:

-- MAX, HIGH, and MEDIUM use a 352-bit matrix as shown below.  Columns are PHI and rows are
-- Pt sub bins and curvature.  Individual CFT track equations are mapped into matrix elements
-- by the track equation VHDL file.

-- The 44x8 matrix is sub-divided into 16 1x22 blocks.  Blocks are
-- assigned a priority, with the lower number being the highest priority.  At each crossing
-- the backend pipeline determines which blocks contain one or more tracks.  The first seven
-- blocks containing one or more tracks are sent down the pipeline sequentially.

-- As the block progresses down the pipeline it is scanned from the "left" (decreasing phi)
-- and from the right (increasing phi) and the first track found in each direction is evaluated.
-- if there is only one track in the block it is reported.  If two tracks are found in the
-- block the LEFT most track (higher PHI) is reported first, then the RIGHT most track (lower
-- phi) is reported immediately after that.  If a block contains three or more tracks the
-- leftmost and rightmost edge tracks are reported, but the ones in the middle are LOST.

--  44444333333333322222222221111111111000000000
--  43210987654321098765432109876543210987654321    Curvature    Pt-Sub Value
-- +--------------------------------------------+   ----------   ------------
-- |[-------  1  --------][-------- 0 ---------]|   CCW 1          0
-- |[-------  3  --------][-------- 2 ---------]|   CW  0          0
-- |[-------  5  --------][-------- 4 ---------]|   CCW 1          1
-- |[-------  7  --------][-------- 6 ---------]|   CW  0          1
-- |[-------  9  --------][-------- 8 ---------]|   CCW 1          2
-- |[------- 11  --------][------- 10 ---------]|   CW  0          2
-- |[------- 13  --------][------- 12 ---------]|   CCW 1          3
-- |[------- 15  --------][------- 14 ---------]|   CW  0          3
-- +--------------------------------------------+

-- for example, consider an 11-track event which fills up the 
-- matrix like this...

--  4444433333333332222222 2221111111111000000000
--  4321098765432109876543 2109876543210987654321
-- +---------------------------------------------+
-- |...................... .....A.........B......|
-- |.........C............ ......................|
-- |...D.....E.....F...... ......................|
-- |..........G.....H..... ......................|
-- |...................... .........I...J........|
-- |...................... ......................|
-- |........K............. ......................|
-- |...................... ......................|
-- +---------------------------------------------+

-- First, the backend logic determines that blocks 0, 3, 5, 7, 8, and 13 
-- have one or more tracks in them.  Next, block 0 is sent down the pipeline and 
-- tracks A and B are reported.  Next block 3 is sent down the pipline; track
-- C is reported.  Next block 5 is sent down the pipeline; track D and F are 
-- found and reported.  However, track E is *LOST*.  Now, block 7 is sent 
-- down the pipeline; track G and H are reported.

-- So in summary, the following tracks are reported in this order:

--      1. Track A: phi=17, curvature=1, pt_sub=0
--      2. Track B: phi=07, curvature=1, pt_sub=0
--      3. Track C: phi=35, curvature=0, pt_sub=0
--      4. Track D: phi=41, curvature=1, pt_sub=1
--      5. Track F: phi=29, curvature=1, pt_sub=1
--      6. Track G: phi=34, curvature=0, pt_sub=1
--      7. Track H: phi=28, curvature=0, pt_sub=1

-- If less than 7 tracks are found the tracks will be compressed into the beginning
-- few timeslices with no gaps.  The remainder of the clock cycles will be zeroed out.

-- 13-bit trk_out format as follows:

-- bit 12:      set to mark the first track of an event
-- bit 11:      set to mark a valid track
-- bit 10:      curvature bit
-- bit 9..6:    pt_sub bin 
-- bits 5..0:   phi value of the track

-- Total pipeline latency is 8/9 clock cycles.  (If load is asserted for the 1st clock,
-- then the first output track can be read on the 9th clock.)

-- Backend pipeline takes up ~9% of an XCV400.  
-- Runs at 60MHz+ in the slowest speed grade!

library ieee;
use ieee.std_logic_1164.all;

entity backend is
    port (
        matrix_d:   in std_logic_vector(351 downto 0);  -- inputs for the matrix
        clock:      in std_logic;                       -- runs at 53MHz (except when loading)
        reset:      in std_logic;                       -- active low async
        load:       in std_logic;                       -- sync matrix load, active high
-- start is not needed for indiividual Pt Aug-8-04
--        start:      out std_logic;						-- active high, set with first muon track.
        dbl_trk:    out std_logic;   -- this would inhibit isolation bit being set
		  muon_trk:	out	std_logic_vector(9 downto 0);	-- Muon tracks L1 		format=VCEePppppp
        ctoc_trk:   out std_logic_vector(9 downto 0)    -- CTOC tracks L1/L2 	format=VCEePppppp
    );
end backend;

architecture backend_arch of backend is

    -- define registers...

        signal load_reg:            std_logic_vector(5 downto 0);
        -- signal trk12_reg:           std_logic;
        signal start_reg:           std_logic;

    -- define signals...

        signal huge_bus:            std_logic_vector(351 downto 0);
        signal block_select:        std_logic_vector(15 downto 0);
        signal one_block:           std_logic_vector(21 downto 0);
        signal blk_tags:            std_logic_vector(15 downto 0);

        signal left_phi_index:      std_logic_vector(4 downto 0);
        signal right_phi_index:     std_logic_vector(4 downto 0);
        signal phi_base:            std_logic_vector(4 downto 0);

        signal pt_sub:              std_logic_vector(1 downto 0);
        signal valid:               std_logic;
        signal curve:               std_logic;

        signal left_track:          std_logic_vector(9 downto 0);
        signal right_track:         std_logic_vector(9 downto 0);
        signal same:                std_logic;

    -- define components...

        component form_tracks
            port (
                clock:          in  std_logic;
                phi_index_l:    in  std_logic_vector(4 downto 0);
                phi_index_r:    in  std_logic_vector(4 downto 0);
                phi_base:       in  std_logic_vector(4 downto 0);
                valid:          in  std_logic;
                pt_sub:         in  std_logic_vector(1 downto 0);
                curve:          in  std_logic;
				muon_trk:		out	std_logic_vector(9 downto 0);
                left_track:     out std_logic_vector(9 downto 0);
                right_track:    out std_logic_vector(9 downto 0);
                same:           out std_logic
            );
        end component;

        component btag16
            port(
                matrix_d:   in  std_logic_vector(351 downto 0);
                blktag_d:   out std_logic_vector(15 downto 0)
            );
        end component;

        component matrix352
         port (
            d:      in  std_logic_vector(351 downto 0);
            clock:  in  std_logic;
            reset:  in  std_logic;
            load:   in  std_logic;
            q:      out std_logic_vector(351 downto 0)
            );
        end component;
    
        component sprior16
         port (
            din:        in  std_logic_vector(15 downto 0);
            clock:      in  std_logic;
            load:       in  std_logic;
            reset:      in  std_logic;
            dout:       out std_logic_vector(15 downto 0)
            );
        end component;
    
        component mux352
         port (
            d:      in  std_logic_vector(351 downto 0);
            sel:    in  std_logic_vector(15 downto 0);
            reset:  in  std_logic;
            clock:  in  std_logic;
            q:      out std_logic_vector(21 downto 0)
            );
        end component;
    
        component blk_id_decode
            port (
                blk_id:     in  std_logic_vector(15 downto 0);
                clock:      in  std_logic;
                pt_sub:     out std_logic_vector(1 downto 0);
                valid:      out std_logic;
                curve:      out std_logic;  
                phi_base:   out std_logic_vector(4 downto 0)   
            );
        end component;
    
        component prior22
            port (
                block_in:   in std_logic_vector(21 downto 0);
                clock:      in std_logic;
                reset:      in std_logic;
                l_index:    out std_logic_vector(4 downto 0);
                r_index:    out std_logic_vector(4 downto 0)
            );
        end component;

        component flex_pipe 
            port (
                clock:      in  std_logic;      -- 53MHz
                start:      in  std_logic;      -- set when first tracks are clocked in.
                same:       in  std_logic;      -- set if tracks are the same.
                trk_a:      in  std_logic_vector(9 downto 0);  -- track A
                trk_b:      in  std_logic_vector(9 downto 0);  -- track B
                ctoc_trk:   out std_logic_vector(9 downto 0)   -- sorted output tracks for CTOC
            );
        end component;
            
begin

    -- combinatorial stuff...

-- start is not needed for indiividual Pt Aug-8-04
--        start       <= start_reg;

    -- sequential stuff...

        -- the last module in the pipeline (flex_pipe) needs to know when the first
        -- track(s) of an event are clocked into it.  This signal is LOAD delayed by 
        -- 5 clock cycles.  load_reg(1) transitions from 0->1 when matrix is loaded.

--        delay_start: process(clock, reset) change to sync reset Aug-08-04
        delay_start: process(clock)
            begin
              if rising_edge(clock) then
				   if (reset='0') then
					  load_reg(5 downto 0) <= "000000";
-- start is not needed for indiividual Pt Aug-8-04
--					start_reg			<= '0';
					else
                    load_reg(5 downto 1)    <= load_reg(4 downto 0);
                    load_reg(0)             <= load;
-- start is not needed for indiividual Pt Aug-8-04
--                    start_reg               <= load_reg(4);
                 end if;
					end if;
        end process;

    -- instantiate components...

        calc_blk_tags: btag16
            port map(
                matrix_d(351 downto 0)  => matrix_d(351 downto 0),
                blktag_d(15 downto 0)   => blk_tags(15 downto 0)
            );

        phi_pt_matrix: matrix352
            port map(
                d(351 downto 0)     => matrix_d(351 downto 0),
                clock               => clock,
                load                => load_reg(0),
                reset               => reset,
                q(351 downto 0)     => huge_bus(351 downto 0)
            );
    
        peel_a_way: sprior16
            port map(
                din(15 downto 0)    => blk_tags(15 downto 0),
                clock               => clock,
                load                => load_reg(0),
                reset               => reset,
                dout(15 downto 0)   => block_select(15 downto 0)
            );
    
        block_mux: mux352
            port map(
                d(351 downto 0)     => huge_bus(351 downto 0),
                sel(15 downto 0)    => block_select(15 downto 0),
                reset               => reset,
                clock               => clock,
                q(21 downto 0)      => one_block(21 downto 0)
            );
    
        block_id_decoder: blk_id_decode
            port map(
                blk_id(15 downto 0) => block_select(15 downto 0),
                clock               => clock,
                pt_sub(1 downto 0)  => pt_sub(1 downto 0),
                valid               => valid,
                curve               => curve,
                phi_base(4 downto 0)=> phi_base(4 downto 0)
            );
    
        dual_pri_encoder: prior22
            port map(
                block_in(21 downto 0)   => one_block(21 downto 0),
                clock                   => clock,
                reset                   => reset,
                l_index(4 downto 0)     => left_phi_index(4 downto 0),
                r_index(4 downto 0)     => right_phi_index(4 downto 0)
            );
    
        track_former: form_tracks
            port map(
                clock                       => clock,
                phi_index_l(4 downto 0)     => left_phi_index(4 downto 0),
                phi_index_r(4 downto 0)     => right_phi_index(4 downto 0),
                phi_base(4 downto 0)        => phi_base(4 downto 0),
                valid                       => valid,
                pt_sub(1 downto 0)          => pt_sub(1 downto 0),
                curve                       => curve,
--				muon_trk(9 downto 0)		=> muon_trk(9 downto 0),
                left_track(9 downto 0)      => left_track(9 downto 0),
                right_track(9 downto 0)     => right_track(9 downto 0),
                same                        => same
            );
muon_trk <= left_track;    
process(clock)
begin
  if(clock'event and clock = '1')then
    if(load_reg(4) = '1')then
      dbl_trk <= left_track(9) and right_track(9) and not same;
	 end if;
  end if;
end process;
        track_sort: flex_pipe 
            port map(
                clock                   => clock,
                start                   => load_reg(4),
                same                    => same,
                trk_a(9 downto 0)       => left_track(9 downto 0),
                trk_b(9 downto 0)       => right_track(9 downto 0),
                ctoc_trk(9 downto 0)    => ctoc_trk(9 downto 0)
            );
    
end backend_arch;
