-- blk_id_decode.vhd
-- jamieson olsen <jamieson@fnal.gov>

--  *** revision history ***

--  28 jun 1999 original design.  used case statement to implement look up tables
--              trouble making 53MHz.
--  13 mar 2000 remove big case statement; implement look up tables with eqns. Faster.
--  19 Dec 2001 update track curvature definition, 

--  03 Jan 2002 get rid of the single "OFFSET" output and break it out into CURVE, 
--              VALID, and PT_SUB() outputs.  MAKE THIS file generic so it can be used
--              for MAX, HIGH, and MED DESIGNS!

--	05 Aug 2002 trim the Pt sub bin back to two bits, not four.
--   7 May 2003 correct curvature definition

--  what duz this do?  A "one hot" blk_id word is supplied to this module.  this module decodes
--  it into a PHI_BASE (0 or 22) and some other stuff:

-- VALID        valid track bit.  Set if the blk_id word has at least 1 set bit in it.
--
-- CURVE        track curvature bit.  set if the track curves in a CCW direction
--              when the detector is viewed "LOOKING NORTH at the SOUTH face of the
--              detector".
--
-- PT_SUB       pt sub-bin. See below.

-- 1x22 blocks are arranged in the 44x8 Matrix as follows:

--  44444333333333322222222221111111111000000000
--  43210987654321098765432109876543210987654321    Curvature    Pt-Sub Value
-- +--------------------------------------------+   ----------   ------------
-- |[-------  1  --------][-------- 0 ---------]|   CCW 1          0
-- |[-------  3  --------][-------- 2 ---------]|   CW  0          0
-- |[-------  5  --------][-------- 4 ---------]|   CCW 1          1
-- |[-------  7  --------][-------- 6 ---------]|   CW  0          1
-- |[-------  9  --------][-------- 8 ---------]|   CCW 1          2
-- |[------- 11  --------][------- 10 ---------]|   CW  0          2
-- |[------- 13  --------][------- 12 ---------]|   CCW 1          3
-- |[------- 15  --------][------- 14 ---------]|   CW  0          3
-- +--------------------------------------------+

library ieee;
use ieee.std_logic_1164.all;

entity blk_id_decode is
    port (
        blk_id:     in  std_logic_vector(15 downto 0);
        clock:      in  std_logic;

        valid:      out std_logic;
        curve:      out std_logic;
        pt_sub:     out std_logic_vector(1 downto 0);
        phi_base:   out std_logic_vector(4 downto 0)    
    );
end blk_id_decode;


architecture block_id_arch of blk_id_decode is

    -- define registers

    signal block_id_reg:    std_logic_vector(15 downto 0);
    signal curve_reg:       std_logic;
    signal valid_reg:       std_logic;
    signal pt_sub_reg:      std_logic_vector(1 downto 0);
    signal phi_base_reg:    std_logic_vector(4 downto 0);

    -- signals

    signal temp_sub:        std_logic_vector(1 downto 0);
    signal temp_phi:        std_logic_vector(4 downto 0);
    signal temp_val:        std_logic;
    signal temp_curve:      std_logic;
    -- signal temp_offset:     std_logic_vector(2 downto 0);  -- 24 Feb 2004, XST cleanup


 begin


    -- combinatorial stuff

    curve                   <= curve_reg;
    valid                   <= valid_reg;
    pt_sub(1 downto 0)      <= pt_sub_reg(1 downto 0);
    phi_base(4 downto 0)    <= phi_base_reg(4 downto 0);

    temp_val                <=  block_id_reg(15) or block_id_reg(14) or
                                block_id_reg(13) or block_id_reg(12) or
                                block_id_reg(11) or block_id_reg(10) or
                                block_id_reg( 9) or block_id_reg( 8) or
                                block_id_reg( 7) or block_id_reg( 6) or
                                block_id_reg( 5) or block_id_reg( 4) or
                                block_id_reg( 3) or block_id_reg( 2) or
                                block_id_reg( 1) or block_id_reg( 0);                               

    temp_curve              <=  block_id_reg(13) or block_id_reg(12) or
                                block_id_reg( 9) or block_id_reg( 8) or
                                block_id_reg( 5) or block_id_reg( 4) or
                                block_id_reg( 1) or block_id_reg( 0);

    temp_sub(1 downto 0) <=     "00" when (block_id_reg( 0)='1' or block_id_reg( 1)='1') else 
                                "00" when (block_id_reg( 2)='1' or block_id_reg( 3)='1') else 
                                "01" when (block_id_reg( 4)='1' or block_id_reg( 5)='1') else 
                                "01" when (block_id_reg( 6)='1' or block_id_reg( 7)='1') else 
                                "10" when (block_id_reg( 8)='1' or block_id_reg( 9)='1') else 
                                "10" when (block_id_reg(10)='1' or block_id_reg(11)='1') else
                                "11";

    temp_phi(4 downto 0) <=     "10110" when (block_id_reg( 1)='1' or block_id_reg( 3)='1') else
                                "10110" when (block_id_reg( 5)='1' or block_id_reg( 7)='1') else
                                "10110" when (block_id_reg( 9)='1' or block_id_reg(11)='1') else
                                "10110" when (block_id_reg(13)='1' or block_id_reg(15)='1') else
                                "00000";

    -- sequential stuff

        process(clock)
         begin
            if rising_edge(clock) then
                block_id_reg(15 downto 0)   <= blk_id(15 downto 0);
                valid_reg                   <= temp_val;        
                curve_reg                   <= temp_curve;
                pt_sub_reg(1 downto 0)      <= temp_sub(1 downto 0);
                phi_base_reg(4 downto 0)    <= temp_phi(4 downto 0);
            end if;
        end process;
    
end block_id_arch;
