-- stretchy pipelined track sorter
-- jamieson olsen 3 Jan 2002

-- accepts two 12-bit track words on each clk cycle.  If tracks are different, it sends 
-- out A first and then B.  If they are the same only A is sent.  Does not use a stack 
-- like the old version, so there is no track scrambling.  
-- For example, if the following tracks are found:

-- A: 801, 803, 804, 805
-- B: 802, 803, 804, 806

-- Q: 1801, 0802, 0803, 0804, 1805, 0806

-- *** revision history ***

-- 3 Jan 2002: created and tested.
-- 5 Aug 2002: reduce input track from 11 bits to 9 bits.  Format VCEePppppp.

library ieee;
use ieee.std_logic_1164.all;

entity flex_pipe is
 	port (
		clock:		in 	std_logic;		-- 53MHz
		start:		in	std_logic;		-- set when first tracks are clocked in.
		same:		in	std_logic;	 	-- set if tracks are the same.

		trk_a:		in 	std_logic_vector(9 downto 0);	-- track A
		trk_b:		in	std_logic_vector(9 downto 0);	-- track B
		ctoc_trk:	out	std_logic_vector(9 downto 0)	-- sorted output tracks for CTOC
	);
end flex_pipe;

architecture stretch_arch of flex_pipe is

	-- define registers...

		signal dly_reg:		std_logic_vector(1 downto 0);

	-- define signals...
		signal logic:		std_logic_vector( 1 downto 0);
		signal rama_din:	std_logic_vector(12 downto 0);
		signal ramb_din:	std_logic_vector(12 downto 0);
		signal rama_dout:	std_logic_vector(12 downto 0);
		signal ramb_dout:	std_logic_vector(12 downto 0);
		signal wptr_a:		std_logic_vector( 3 downto 0);
		signal wptr_b:		std_logic_vector( 3 downto 0);
		signal read_ptr:	std_logic_vector( 3 downto 0);

	-- declare components...

		component dpr_load is
		 	port (
				trk_a:		in 	std_logic_vector(9 downto 0);
				trk_b:		in	std_logic_vector(9 downto 0);
				same:		in 	std_logic;
				clock:		in 	std_logic;
				start:		in	std_logic;						-- active high, set for 1st word of the frame
				data_a:		out	std_logic_vector(10 downto 0);	-- data going into rams...
				data_b:		out	std_logic_vector(10 downto 0);
				ptr_a:		out	std_logic_vector(3 downto 0);	-- address pointers
				ptr_b:		out	std_logic_vector(3 downto 0)
			);
		end component;

		component dp_ram is
		 	port (
				wclk:		in	std_logic;						-- sync write clk
				we:			in	std_logic;						-- active high
				addra:		in	std_logic_vector( 3 downto 0);	-- write port address
				din:		in	std_logic_vector(10 downto 0);		
				addrb:		in	std_logic_vector( 3 downto 0);	-- read port address
				dout:		out	std_logic_vector(10 downto 0)	-- read data
			);
		end component;

		component dpr_read is
		 	port (
				clock:		in	std_logic;						-- 53MHz
				clear:		in	std_logic;						-- active high
				data_a:		in 	std_logic_vector(10 downto 0);	-- dout from RAMs
				data_b:		in	std_logic_vector(10 downto 0);
				ptr:		out	std_logic_vector( 3 downto 0);	-- address pointer to RAMs
				ctoc_trk:	out	std_logic_vector( 9 downto 0)	-- track out for CTOC L1/l2
			);
		end component;
		
begin

	-- combinatorial stuff...

		logic(1) <= '1';
		logic(0) <= '0';


	-- sequential stuff...

		-- the front end needs to know the first track(s) of an event.  This is done with 
		-- with the START input.  Likewise, the backend needs to know when it's safe to clear the 
		-- read pointer and begin pulling data out of the rams.  CLEAR is a copy of START delayed by
		-- TWO clock cycles...

		clr_dly: process(clock)
		 begin
			if rising_edge(clock) then
				dly_reg(1) 	<= dly_reg(0);
				dly_reg(0)	<= start;
			end if;
		end process;

	-- instantiate components...

		write_module: dpr_load
		 	port map(
				trk_a(9 downto 0) 	=> trk_a(9 downto 0),
				trk_b(9 downto 0) 	=> trk_b(9 downto 0),
				same				=> same,
				clock				=> clock,
				start				=> start,
				data_a(10 downto 0)	=> rama_din(10 downto 0),
				data_b(10 downto 0)	=> ramb_din(10 downto 0),
				ptr_a(3 downto 0)	=> wptr_a(3 downto 0),
				ptr_b(3 downto 0)	=> wptr_b(3 downto 0)
			);

		ram_a: dp_ram
		 	port map(
				wclk				=> clock,
				we					=> logic(1),
				addra(3 downto 0)	=> wptr_a(3 downto 0),
				din(10 downto 0)	=> rama_din(10 downto 0),
				addrb( 3 downto 0)	=> read_ptr(3 downto 0),
				dout(10 downto 0)	=> rama_dout(10 downto 0)
			);

		ram_b: dp_ram
		 	port map(
				wclk				=> clock,
				we					=> logic(1),
				addra(3 downto 0)	=> wptr_b(3 downto 0),
				din(10 downto 0)	=> ramb_din(10 downto 0),
				addrb( 3 downto 0)	=> read_ptr(3 downto 0),
				dout(10 downto 0)	=> ramb_dout(10 downto 0)
			);

		read_module: dpr_read
		 	port map(
				clock					=> clock,
				clear					=> dly_reg(1),
				data_a(10 downto 0)		=> rama_dout(10 downto 0),
				data_b(10 downto 0)		=> ramb_dout(10 downto 0),
				ptr(3  downto 0)		=> read_ptr(3 downto 0),
				ctoc_trk(9 downto 0)	=> ctoc_trk(9 downto 0)
			);
		

end stretch_arch;
